@@ -50,7 +50,6 @@ unsigned long conf_bgp_debug_keepalive;
 unsigned long conf_bgp_debug_update;
 unsigned long conf_bgp_debug_bestpath;
 unsigned long conf_bgp_debug_zebra;
-unsigned long conf_bgp_debug_allow_martians;
 unsigned long conf_bgp_debug_nht;
 unsigned long conf_bgp_debug_update_groups;
 unsigned long conf_bgp_debug_vpn;
@@ -61,6 +60,7 @@ unsigned long conf_bgp_debug_graceful_restart;
 unsigned long conf_bgp_debug_evpn_mh;
 unsigned long conf_bgp_debug_bfd;
 unsigned long conf_bgp_debug_cond_adv;
+unsigned long conf_bgp_debug_aggregate;
 
 unsigned long term_bgp_debug_as4;
 unsigned long term_bgp_debug_neighbor_events;
@@ -71,7 +71,6 @@ unsigned long term_bgp_debug_keepalive;
 unsigned long term_bgp_debug_update;
 unsigned long term_bgp_debug_bestpath;
 unsigned long term_bgp_debug_zebra;
-unsigned long term_bgp_debug_allow_martians;
 unsigned long term_bgp_debug_nht;
 unsigned long term_bgp_debug_update_groups;
 unsigned long term_bgp_debug_vpn;
@@ -82,6 +81,7 @@ unsigned long term_bgp_debug_graceful_restart;
 unsigned long term_bgp_debug_evpn_mh;
 unsigned long term_bgp_debug_bfd;
 unsigned long term_bgp_debug_cond_adv;
+unsigned long term_bgp_debug_aggregate;
 
 struct list *bgp_debug_neighbor_events_peers = NULL;
 struct list *bgp_debug_keepalive_peers = NULL;
@@ -90,6 +90,7 @@ struct list *bgp_debug_update_in_peers = NULL;
 struct list *bgp_debug_update_prefixes = NULL;
 struct list *bgp_debug_bestpath_prefixes = NULL;
 struct list *bgp_debug_zebra_prefixes = NULL;
+struct list *bgp_debug_aggregate_prefixes;
 
 /* messages for BGP-4 status */
 const struct message bgp_status_msg[] = {{Idle, "Idle"},
@@ -116,6 +117,7 @@ static const struct message bgp_notify_msg[] = {
 	{BGP_NOTIFY_FSM_ERR, "Neighbor Events Error"},
 	{BGP_NOTIFY_CEASE, "Cease"},
 	{BGP_NOTIFY_ROUTE_REFRESH_ERR, "ROUTE-REFRESH Message Error"},
+	{BGP_NOTIFY_SEND_HOLD_ERR, "Send Hold Timer Expired"},
 	{0}};
 
 static const struct message bgp_notify_head_msg[] = {
@@ -450,6 +452,10 @@ bool bgp_dump_attr(struct attr *attr, char *buf, size_t size)
 			 ", extcommunity %s",
 			 ecommunity_str(bgp_attr_get_ecommunity(attr)));
 
+	if (CHECK_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_IPV6_EXT_COMMUNITIES)))
+		snprintf(buf + strlen(buf), size - strlen(buf), ", ipv6-extcommunity %s",
+			 ecommunity_str(bgp_attr_get_ipv6_ecommunity(attr)));
+
 	if (CHECK_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_ATOMIC_AGGREGATE)))
 		snprintf(buf + strlen(buf), size - strlen(buf),
 			 ", atomic-aggregate");
@@ -515,6 +521,7 @@ const char *bgp_notify_subcode_str(char code, char subcode)
 		return lookup_msg(bgp_notify_update_msg, subcode,
 				  "Unrecognized Error Subcode");
 	case BGP_NOTIFY_HOLD_ERR:
+	case BGP_NOTIFY_SEND_HOLD_ERR:
 		break;
 	case BGP_NOTIFY_FSM_ERR:
 		return lookup_msg(bgp_notify_fsm_msg, subcode,
@@ -1808,6 +1815,107 @@ DEFPY (no_debug_bgp_zebra_prefix,
 	return CMD_SUCCESS;
 }
 
+/* debug bgp aggregate */
+DEFPY (debug_bgp_aggregate,
+       debug_bgp_aggregate_cmd,
+       "debug bgp aggregate",
+       DEBUG_STR
+       BGP_STR
+       "BGP aggregate\n")
+{
+	if (vty->node == CONFIG_NODE)
+		DEBUG_ON(aggregate, AGGREGATE);
+	else {
+		TERM_DEBUG_ON(aggregate, AGGREGATE);
+		vty_out(vty, "BGP aggregate debugging is on\n");
+	}
+	return CMD_SUCCESS;
+}
+
+DEFPY (no_debug_bgp_aggregate,
+       no_debug_bgp_aggregate_cmd,
+       "no debug bgp aggregate",
+       NO_STR
+       DEBUG_STR
+       BGP_STR
+       "BGP aggregate\n")
+{
+	bgp_debug_list_free(bgp_debug_aggregate_prefixes);
+
+	if (vty->node == CONFIG_NODE)
+		DEBUG_OFF(aggregate, AGGREGATE);
+	else {
+		TERM_DEBUG_OFF(aggregate, AGGREGATE);
+		vty_out(vty, "BGP aggregate debugging is off\n");
+	}
+	return CMD_SUCCESS;
+}
+
+DEFPY (debug_bgp_aggregate_prefix,
+       debug_bgp_aggregate_prefix_cmd,
+       "debug bgp aggregate prefix <A.B.C.D/M|X:X::X:X/M>$prefix",
+       DEBUG_STR
+       BGP_STR
+       "BGP aggregate\n"
+       "Specify a prefix to debug\n"
+       "IPv4 prefix\n"
+       "IPv6 prefix\n")
+{
+	if (!bgp_debug_aggregate_prefixes)
+		bgp_debug_aggregate_prefixes = list_new();
+
+	if (bgp_debug_list_has_entry(bgp_debug_aggregate_prefixes, NULL, prefix, NULL)) {
+		vty_out(vty, "BGP aggregate debugging is already enabled for %s\n", prefix_str);
+		return CMD_SUCCESS;
+	}
+
+	bgp_debug_list_add_entry(bgp_debug_aggregate_prefixes, NULL, prefix, NULL);
+
+	if (vty->node == CONFIG_NODE)
+		DEBUG_ON(aggregate, AGGREGATE);
+	else {
+		TERM_DEBUG_ON(aggregate, AGGREGATE);
+		vty_out(vty, "BGP aggregate debugging is on for %s\n", prefix_str);
+	}
+
+	return CMD_SUCCESS;
+}
+
+DEFPY (no_debug_bgp_aggregate_prefix,
+       no_debug_bgp_aggregate_prefix_cmd,
+       "no debug bgp aggregate prefix <A.B.C.D/M|X:X::X:X/M>$prefix",
+       NO_STR
+       DEBUG_STR
+       BGP_STR
+       "BGP aggregate\n"
+       "Specify a prefix to debug\n"
+       "IPv4 prefix\n"
+       "IPv6 prefix\n")
+{
+	bool found_prefix = false;
+
+	if (bgp_debug_aggregate_prefixes && !list_isempty(bgp_debug_aggregate_prefixes)) {
+		found_prefix = bgp_debug_list_remove_entry(bgp_debug_aggregate_prefixes, NULL,
+							   (struct prefix *)prefix);
+
+		if (list_isempty(bgp_debug_aggregate_prefixes)) {
+			if (vty->node == CONFIG_NODE)
+				DEBUG_OFF(aggregate, AGGREGATE);
+			else {
+				TERM_DEBUG_OFF(aggregate, AGGREGATE);
+				vty_out(vty, "BGP aggregate debugging is off\n");
+			}
+		}
+	}
+
+	if (found_prefix)
+		vty_out(vty, "BGP aggregate debugging is off for %s\n", prefix_str);
+	else
+		vty_out(vty, "BGP aggregate debugging was not enabled for %s\n", prefix_str);
+
+	return CMD_SUCCESS;
+}
+
 /* debug bgp update-groups */
 DEFUN (debug_bgp_update_groups,
        debug_bgp_update_groups_cmd,
@@ -2163,7 +2271,6 @@ DEFUN (no_debug_bgp,
 	TERM_DEBUG_OFF(as4, AS4_SEGMENT);
 	TERM_DEBUG_OFF(neighbor_events, NEIGHBOR_EVENTS);
 	TERM_DEBUG_OFF(zebra, ZEBRA);
-	TERM_DEBUG_OFF(allow_martians, ALLOW_MARTIANS);
 	TERM_DEBUG_OFF(nht, NHT);
 	TERM_DEBUG_OFF(vpn, VPN_LEAK_FROM_VRF);
 	TERM_DEBUG_OFF(vpn, VPN_LEAK_TO_VRF);
@@ -2236,12 +2343,13 @@ DEFUN_NOSH (show_debugging_bgp,
 		bgp_debug_list_print(vty, "  BGP zebra debugging is on",
 				     bgp_debug_zebra_prefixes);
 
+	if (BGP_DEBUG(aggregate, AGGREGATE))
+		bgp_debug_list_print(vty, "  BGP aggregate debugging is on",
+				     bgp_debug_aggregate_prefixes);
+
 	if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
 		vty_out(vty, "  BGP graceful-restart debugging is on\n");
 
-	if (BGP_DEBUG(allow_martians, ALLOW_MARTIANS))
-		vty_out(vty, "  BGP allow martian next hop debugging is on\n");
-
 	if (BGP_DEBUG(vpn, VPN_LEAK_FROM_VRF))
 		vty_out(vty,
 			"  BGP route leak from vrf to vpn debugging is on\n");
@@ -2354,11 +2462,6 @@ static int bgp_config_write_debug(struct vty *vty)
 		}
 	}
 
-	if (CONF_BGP_DEBUG(allow_martians, ALLOW_MARTIANS)) {
-		vty_out(vty, "debug bgp allow-martians\n");
-		write++;
-	}
-
 	if (CONF_BGP_DEBUG(vpn, VPN_LEAK_FROM_VRF)) {
 		vty_out(vty, "debug bgp vpn leak-from-vrf\n");
 		write++;
@@ -2417,6 +2520,16 @@ static int bgp_config_write_debug(struct vty *vty)
 		write++;
 	}
 
+	if (CONF_BGP_DEBUG(aggregate, AGGREGATE)) {
+		if (!bgp_debug_aggregate_prefixes || list_isempty(bgp_debug_aggregate_prefixes)) {
+			vty_out(vty, "debug bgp aggregate\n");
+			write++;
+		} else {
+			write += bgp_debug_list_conf_print(vty, "debug bgp aggregate prefix",
+							   bgp_debug_aggregate_prefixes);
+		}
+	}
+
 	if (hook_call(bgp_hook_config_write_debug, vty, true))
 		write++;
 
@@ -2490,6 +2603,16 @@ void bgp_debug_init(void)
 	install_element(ENABLE_NODE, &no_debug_bgp_zebra_prefix_cmd);
 	install_element(CONFIG_NODE, &no_debug_bgp_zebra_prefix_cmd);
 
+	/* debug bgp aggregate prefix A.B.C.D/M */
+	install_element(ENABLE_NODE, &debug_bgp_aggregate_cmd);
+	install_element(CONFIG_NODE, &debug_bgp_aggregate_cmd);
+	install_element(ENABLE_NODE, &no_debug_bgp_aggregate_cmd);
+	install_element(CONFIG_NODE, &no_debug_bgp_aggregate_cmd);
+	install_element(ENABLE_NODE, &debug_bgp_aggregate_prefix_cmd);
+	install_element(CONFIG_NODE, &debug_bgp_aggregate_prefix_cmd);
+	install_element(ENABLE_NODE, &no_debug_bgp_aggregate_prefix_cmd);
+	install_element(CONFIG_NODE, &no_debug_bgp_aggregate_prefix_cmd);
+
 	install_element(ENABLE_NODE, &no_debug_bgp_as4_cmd);
 	install_element(CONFIG_NODE, &no_debug_bgp_as4_cmd);
 	install_element(ENABLE_NODE, &no_debug_bgp_as4_segment_cmd);
@@ -2567,7 +2690,7 @@ static int bgp_debug_per_prefix(const struct prefix *p,
 	struct bgp_debug_filter *filter;
 	struct listnode *node, *nnode;
 
-	if (term_bgp_debug_type & BGP_DEBUG_TYPE) {
+	if (CHECK_FLAG(term_bgp_debug_type, BGP_DEBUG_TYPE)) {
 		/* We are debugging all prefixes so return true */
 		if (!per_prefix_list || list_isempty(per_prefix_list))
 			return 1;
@@ -2600,7 +2723,7 @@ static bool bgp_debug_per_peer(char *host, const struct prefix *p,
 	struct bgp_debug_filter *filter;
 	struct listnode *node, *nnode;
 
-	if (term_bgp_debug_type & BGP_DEBUG_TYPE) {
+	if (CHECK_FLAG(term_bgp_debug_type, BGP_DEBUG_TYPE)) {
 		/* We are debugging all peers so return true */
 		if (!per_peer_list || list_isempty(per_peer_list))
 			return true;
@@ -2719,10 +2842,21 @@ bool bgp_debug_zebra(const struct prefix *p)
 	return false;
 }
 
+bool bgp_debug_aggregate(const struct prefix *p)
+{
+	if (BGP_DEBUG(aggregate, AGGREGATE)) {
+		if (bgp_debug_per_prefix(p, term_bgp_debug_aggregate, BGP_DEBUG_AGGREGATE,
+					 bgp_debug_aggregate_prefixes))
+			return true;
+	}
+
+	return false;
+}
+
 const char *bgp_debug_rdpfxpath2str(afi_t afi, safi_t safi,
 				    const struct prefix_rd *prd,
 				    union prefixconstptr pu,
-				    mpls_label_t *label, uint32_t num_labels,
+				    mpls_label_t *label, uint8_t num_labels,
 				    int addpath_valid, uint32_t addpath_id,
 				    struct bgp_route_evpn *overlay_index,
 				    char *str, int size)
