@@ -199,7 +199,7 @@ static int read_ibuf_work(struct peer_connection *connection)
 	assert(ringbuf_get(ibw, pkt->data, pktsize) == pktsize);
 	stream_set_endp(pkt, pktsize);
 
-	frrtrace(2, frr_bgp, packet_read, connection->peer, pkt);
+	frrtrace(2, frr_bgp, packet_read, connection, pkt);
 	frr_with_mutex (&connection->io_mtx) {
 		stream_fifo_push(connection->ibuf, pkt);
 	}
@@ -512,7 +512,13 @@ static uint16_t bgp_read(struct peer_connection *connection, int *code_p)
 
 	readsize = MIN(ibuf_work_space, sizeof(ibuf_scratch));
 
+#ifdef __clang_analyzer__
+	/* clang-SA doesn't want you to call read() while holding a mutex */
+	(void)readsize;
+	nbytes = 0;
+#else
 	nbytes = read(connection->fd, ibuf_scratch, readsize);
+#endif
 
 	/* EAGAIN or EWOULDBLOCK; come back later */
 	if (nbytes < 0 && ERRNO_IO_RETRY(errno)) {
