@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * BGP pbr
  * Copyright (C) 6WIND
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __BGP_PBR_H__
 #define __BGP_PBR_H__
@@ -147,7 +134,7 @@ struct bgp_pbr_entry_main {
 
 struct bgp_pbr_interface {
 	RB_ENTRY(bgp_pbr_interface) id_entry;
-	char name[INTERFACE_NAMSIZ];
+	char name[IFNAMSIZ];
 };
 
 RB_HEAD(bgp_pbr_interface_head, bgp_pbr_interface);
