@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * EIGRPd Finite State Machine (DUAL).
  * Copyright (C) 2013-2014
@@ -8,22 +9,6 @@
  *   Peter Orsag
  *   Peter Paluch
  *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
- *
  * This file contains functions for executing logic of finite state machine
  *
  *                                +------------ +
@@ -68,8 +53,8 @@
  */
 
 #include <zebra.h>
-#include <thread.h>
 
+#include "frrevent.h"
 #include "prefix.h"
 #include "table.h"
 #include "memory.h"
