@@ -1,23 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_vty_fabricd.c
  *
  * This file contains the CLI that is specific to OpenFabric
  *
  * Copyright (C) 2018        Christian Franke, for NetDEF, Inc.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public Licenseas published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
@@ -118,7 +105,7 @@ static void lsp_print_flooding(struct vty *vty, struct isis_lsp *lsp,
 	char lspid[255];
 	char buf[MONOTIME_STRLEN];
 
-	lspid_print(lsp->hdr.lsp_id, lspid, true, true, isis);
+	lspid_print(lsp->hdr.lsp_id, lspid, sizeof(lspid), true, true, isis);
 	vty_out(vty, "Flooding information for %s\n", lspid);
 
 	if (!lsp->flooding_neighbors[TX_LSP_NORMAL]) {
@@ -187,7 +174,7 @@ DEFUN (show_lsp_flooding,
 		vty_out(vty, "Area %s:\n",
 			area->area_tag ? area->area_tag : "null");
 		if (lspid) {
-			lsp = lsp_for_arg(head, lspid, isis);
+			lsp = lsp_for_sysid(head, lspid, isis);
 			if (lsp)
 				lsp_print_flooding(vty, lsp, isis);
 			continue;
@@ -229,18 +216,10 @@ DEFUN (ip_router_isis,
 
 	area = isis_area_lookup(area_tag, VRF_DEFAULT);
 	if (!area)
-		area = isis_area_create(area_tag, VRF_DEFAULT_NAME);
+		isis_area_create(area_tag, VRF_DEFAULT_NAME);
 
-	if (!circuit || !circuit->area) {
-		circuit = isis_circuit_create(area, ifp);
-
-		if (circuit->state != C_STATE_CONF
-		    && circuit->state != C_STATE_UP) {
-			vty_out(vty,
-				"Couldn't bring up interface, please check log.\n");
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-	}
+	if (!circuit)
+		circuit = isis_circuit_new(ifp, area_tag);
 
 	bool ip = circuit->ip_router, ipv6 = circuit->ipv6_router;
 	if (af[2] != '\0')
@@ -288,7 +267,7 @@ DEFUN (no_ip_router_isis,
 		return CMD_ERR_NO_MATCH;
 	}
 
-	circuit = circuit_lookup_by_ifp(ifp, area->circuit_list);
+	circuit = circuit_scan_by_ifp(ifp);
 	if (!circuit) {
 		vty_out(vty, "ISIS is not enabled on circuit %s\n", ifp->name);
 		return CMD_ERR_NO_MATCH;
@@ -301,6 +280,10 @@ DEFUN (no_ip_router_isis,
 		ip = false;
 
 	isis_circuit_af_set(circuit, ip, ipv6);
+
+	if (!ip && !ipv6)
+		isis_circuit_del(circuit);
+
 	return CMD_SUCCESS;
 }
 
@@ -315,13 +298,11 @@ DEFUN (isis_bfd,
 	if (!circuit)
 		return CMD_ERR_NO_MATCH;
 
-	if (circuit->bfd_info
-	    && CHECK_FLAG(circuit->bfd_info->flags, BFD_FLAG_PARAM_CFG)) {
+	if (circuit->bfd_config.enabled)
 		return CMD_SUCCESS;
-	}
 
-	isis_bfd_circuit_param_set(circuit, BFD_DEF_MIN_RX, BFD_DEF_MIN_TX,
-				   BFD_DEF_DETECT_MULT, NULL, true);
+	circuit->bfd_config.enabled = true;
+	isis_bfd_circuit_cmd(circuit);
 
 	return CMD_SUCCESS;
 }
@@ -339,11 +320,12 @@ DEFUN (no_isis_bfd,
 	if (!circuit)
 		return CMD_ERR_NO_MATCH;
 
-	if (!circuit->bfd_info)
+	if (!circuit->bfd_config.enabled)
 		return CMD_SUCCESS;
 
-	isis_bfd_circuit_cmd(circuit, ZEBRA_BFD_DEST_DEREGISTER);
-	bfd_info_free(&circuit->bfd_info);
+	circuit->bfd_config.enabled = false;
+	isis_bfd_circuit_cmd(circuit);
+
 	return CMD_SUCCESS;
 }
 
