@@ -16,14 +16,8 @@ extern "C" {
 #endif
 
 #define BFD_DEF_MIN_RX 300
-#define BFD_MIN_MIN_RX 50
-#define BFD_MAX_MIN_RX 60000
 #define BFD_DEF_MIN_TX 300
-#define BFD_MIN_MIN_TX 50
-#define BFD_MAX_MIN_TX 60000
 #define BFD_DEF_DETECT_MULT 3
-#define BFD_MIN_DETECT_MULT 2
-#define BFD_MAX_DETECT_MULT 255
 
 #define BFD_STATUS_UNKNOWN    (1 << 0) /* BFD session status never received */
 #define BFD_STATUS_DOWN       (1 << 1) /* BFD session status is down */
@@ -32,6 +26,8 @@ extern "C" {
 
 #define BFD_PROFILE_NAME_LEN 64
 
+#define BFD_NAME_SIZE 255
+
 const char *bfd_get_status_str(int status);
 
 extern void bfd_client_sendmsg(struct zclient *zclient, int command,
@@ -415,6 +411,8 @@ struct bfd_session_arg {
 	uint32_t min_tx;
 	/** Detection multiplier. */
 	uint32_t detection_multiplier;
+	/* bfd session name*/
+	char bfd_name[BFD_NAME_SIZE + 1];
 };
 
 /**
@@ -464,6 +462,8 @@ extern bool bfd_protocol_integration_shutting_down(void);
 extern int bfd_nht_update(const struct prefix *match,
 			  const struct zapi_route *route);
 
+extern bool bfd_session_is_down(const struct bfd_session_params *session);
+
 #ifdef __cplusplus
 }
 #endif
