@@ -17,6 +17,7 @@
 #include <lib/version.h>
 
 #include "command.h"
+#include "debug.h"
 #include "frrstr.h"
 #include "memory.h"
 #include "log.h"
@@ -678,6 +679,21 @@ vector cmd_describe_command(vector vline, struct vty *vty, int *status)
 
 static struct list *varhandlers = NULL;
 
+static int __add_key_comp(const struct lyd_node *dnode, void *arg)
+{
+	const char *key_value = yang_dnode_get_string(dnode, NULL);
+
+	vector_set((vector)arg, XSTRDUP(MTYPE_COMPLETION, key_value));
+
+	return YANG_ITER_CONTINUE;
+}
+
+static void __get_list_keys(vector comps, const char *xpath)
+{
+	yang_dnode_iterate(__add_key_comp, comps,
+			   vty_shared_candidate_config->dnode, "%s", xpath);
+}
+
 void cmd_variable_complete(struct cmd_token *token, const char *arg,
 			   vector comps)
 {
@@ -694,7 +710,10 @@ void cmd_variable_complete(struct cmd_token *token, const char *arg,
 		if (cvh->varname && (!token->varname
 				     || strcmp(cvh->varname, token->varname)))
 			continue;
-		cvh->completions(tmpcomps, token);
+		if (cvh->xpath)
+			__get_list_keys(tmpcomps, cvh->xpath);
+		if (cvh->completions)
+			cvh->completions(tmpcomps, token);
 		break;
 	}
 
@@ -753,7 +772,7 @@ void cmd_variable_handler_register(const struct cmd_variable_handler *cvh)
 	if (!varhandlers)
 		return;
 
-	for (; cvh->completions; cvh++)
+	for (; cvh->completions || cvh->xpath; cvh++)
 		listnode_add(varhandlers, (void *)cvh);
 }
 
@@ -1435,7 +1454,7 @@ DEFUN (config_help,
        "Description of the interactive help system\n")
 {
 	vty_out(vty,
-		"Quagga VTY provides advanced help feature.  When you need help,\n\
+		"FRR VTY provides advanced help feature.  When you need help,\n\
 anytime at the command line please press '?'.\n\
 \n\
 If nothing matches, the help list will be empty and you must backup\n\
@@ -2445,8 +2464,7 @@ const char *host_config_get(void)
 void cmd_show_lib_debugs(struct vty *vty)
 {
 	route_map_show_debug(vty);
-	mgmt_debug_be_client_show_debug(vty);
-	mgmt_debug_fe_client_show_debug(vty);
+	debug_status_write(vty);
 }
 
 void install_default(enum node_type node)
