@@ -111,6 +111,11 @@ static struct cpu_event_history *cpu_records_get(struct event_loop *loop,
 	return res;
 }
 
+static void cpu_records_clear(struct cpu_event_history *p)
+{
+	memset(p->_clear_begin, 0, p->_clear_end - p->_clear_begin);
+}
+
 static void cpu_records_free(struct cpu_event_history **p)
 {
 	XFREE(MTYPE_EVENT_STATS, *p);
@@ -250,20 +255,15 @@ static void cpu_record_clear(uint8_t filter)
 		for (ALL_LIST_ELEMENTS_RO(masters, ln, m)) {
 			frr_with_mutex (&m->mtx) {
 				struct cpu_event_history *item;
-				struct cpu_records_head old[1];
 
-				cpu_records_init(old);
-				cpu_records_swap_all(old, m->cpu_records);
-
-				while ((item = cpu_records_pop(old))) {
+				/* it isn't possible to free the memory here
+				 * because some of these will be in use (e.g.
+				 * the one we're currently running in!)
+				 */
+				frr_each (cpu_records, m->cpu_records, item) {
 					if (item->types & filter)
-						cpu_records_free(&item);
-					else
-						cpu_records_add(m->cpu_records,
-								item);
+						cpu_records_clear(item);
 				}
-
-				cpu_records_fini(old);
 			}
 		}
 	}
@@ -304,9 +304,6 @@ static uint8_t parse_filter(const char *filterstr)
 	return filter;
 }
 
-#if CONFDATE > 20240707
-	CPP_NOTICE("Remove `show thread ...` commands")
-#endif
 DEFUN_NOSH (show_event_cpu,
             show_event_cpu_cmd,
             "show event cpu [FILTER]",
@@ -332,14 +329,6 @@ DEFUN_NOSH (show_event_cpu,
 	return CMD_SUCCESS;
 }
 
-ALIAS(show_event_cpu,
-      show_thread_cpu_cmd,
-      "show thread cpu [FILTER]",
-      SHOW_STR
-      "Thread information\n"
-      "Thread CPU usage\n"
-      "Display filter (rwtex)\n")
-
 DEFPY (service_cputime_stats,
        service_cputime_stats_cmd,
        "[no] service cputime-stats",
@@ -440,19 +429,12 @@ DEFUN_NOSH (show_event_poll,
 	return CMD_SUCCESS;
 }
 
-ALIAS(show_event_poll,
-      show_thread_poll_cmd,
-      "show thread poll",
-      SHOW_STR
-      "Thread information\n"
-      "Show poll FD's and information\n")
-
-DEFUN (clear_thread_cpu,
-       clear_thread_cpu_cmd,
-       "clear thread cpu [FILTER]",
+DEFUN (clear_event_cpu,
+       clear_event_cpu_cmd,
+       "clear event cpu [FILTER]",
        "Clear stored data in all pthreads\n"
-       "Thread information\n"
-       "Thread CPU usage\n"
+       "Event information\n"
+       "Event CPU usage\n"
        "Display filter (rwtexb)\n")
 {
 	uint8_t filter = (uint8_t)-1U;
@@ -507,26 +489,16 @@ DEFPY_NOSH (show_event_timers,
 	return CMD_SUCCESS;
 }
 
-ALIAS(show_event_timers,
-      show_thread_timers_cmd,
-      "show thread timers",
-      SHOW_STR
-      "Thread information\n"
-      "Show all timers and how long they have in the system\n")
-
 void event_cmd_init(void)
 {
-	install_element(VIEW_NODE, &show_thread_cpu_cmd);
 	install_element(VIEW_NODE, &show_event_cpu_cmd);
-	install_element(VIEW_NODE, &show_thread_poll_cmd);
 	install_element(VIEW_NODE, &show_event_poll_cmd);
-	install_element(ENABLE_NODE, &clear_thread_cpu_cmd);
+	install_element(ENABLE_NODE, &clear_event_cpu_cmd);
 
 	install_element(CONFIG_NODE, &service_cputime_stats_cmd);
 	install_element(CONFIG_NODE, &service_cputime_warning_cmd);
 	install_element(CONFIG_NODE, &service_walltime_warning_cmd);
 
-	install_element(VIEW_NODE, &show_thread_timers_cmd);
 	install_element(VIEW_NODE, &show_event_timers_cmd);
 }
 /* CLI end ------------------------------------------------------------------ */
@@ -571,8 +543,9 @@ struct event_loop *event_master_create(const char *name)
 	}
 
 	if (rv->fd_limit > STUPIDLY_LARGE_FD_SIZE) {
-		zlog_warn("FD Limit set: %u is stupidly large.  Is this what you intended?  Consider using --limit-fds also limiting size to %u",
-			  rv->fd_limit, STUPIDLY_LARGE_FD_SIZE);
+		if (frr_is_daemon())
+			zlog_warn("FD Limit set: %u is stupidly large.  Is this what you intended?  Consider using --limit-fds also limiting size to %u",
+				  rv->fd_limit, STUPIDLY_LARGE_FD_SIZE);
 
 		rv->fd_limit = STUPIDLY_LARGE_FD_SIZE;
 	}
@@ -684,24 +657,6 @@ static void thread_array_free(struct event_loop *m, struct event **thread_array)
 	XFREE(MTYPE_EVENT_POLL, thread_array);
 }
 
-/*
- * event_master_free_unused
- *
- * As threads are finished with they are put on the
- * unuse list for later reuse.
- * If we are shutting down, Free up unused threads
- * So we can see if we forget to shut anything off
- */
-void event_master_free_unused(struct event_loop *m)
-{
-	frr_with_mutex (&m->mtx) {
-		struct event *t;
-
-		while ((t = event_list_pop(&m->unuse)))
-			thread_free(m, t);
-	}
-}
-
 /* Stop thread scheduler. */
 void event_master_free(struct event_loop *m)
 {
@@ -808,7 +763,6 @@ static struct event *thread_get(struct event_loop *m, uint8_t type,
 		thread = XCALLOC(MTYPE_THREAD, sizeof(struct event));
 		/* mutex only needs to be initialized at struct creation. */
 		pthread_mutex_init(&thread->mtx, NULL);
-		m->alloc++;
 	}
 
 	thread->type = type;
@@ -847,10 +801,6 @@ static struct event *thread_get(struct event_loop *m, uint8_t type,
 
 static void thread_free(struct event_loop *master, struct event *thread)
 {
-	/* Update statistics. */
-	assert(master->alloc > 0);
-	master->alloc--;
-
 	/* Free allocated resources. */
 	pthread_mutex_destroy(&thread->mtx);
 	XFREE(MTYPE_THREAD, thread);
@@ -994,7 +944,7 @@ void _event_add_read_write(const struct xref_eventsched *xref,
 		 * if we already have a pollfd for our file descriptor, find and
 		 * use it
 		 */
-		for (nfds_t i = 0; i < m->handler.pfdcount; i++)
+		for (nfds_t i = 0; i < m->handler.pfdcount; i++) {
 			if (m->handler.pfds[i].fd == fd) {
 				queuepos = i;
 
@@ -1008,6 +958,15 @@ void _event_add_read_write(const struct xref_eventsched *xref,
 #endif
 				break;
 			}
+			/*
+			 * We are setting the fd = -1 for the
+			 * case when a read/write event is going
+			 * away.  if we find a -1 we can stuff it
+			 * into that spot, so note it
+			 */
+			if (m->handler.pfds[i].fd == -1 && queuepos == m->handler.pfdcount)
+				queuepos = i;
+		}
 
 		/* make sure we have room for this fd + pipe poker fd */
 		assert(queuepos + 1 < m->handler.pfdsize);
@@ -1284,6 +1243,14 @@ static void cancel_arg_helper(struct event_loop *master,
 	for (i = 0; i < master->handler.pfdcount;) {
 		pfd = master->handler.pfds + i;
 
+		/*
+		 * Skip this spot, nothing here to see
+		 */
+		if (pfd->fd == -1) {
+			i++;
+			continue;
+		}
+
 		if (pfd->events & POLLIN)
 			t = master->read[pfd->fd];
 		else
@@ -1605,6 +1572,12 @@ static int thread_process_io_helper(struct event_loop *m, struct event *thread,
 	 * we should.
 	 */
 	m->handler.pfds[pos].events &= ~(state);
+	/*
+	 * ppoll man page says that a fd of -1 causes the particular
+	 * array item to be skipped.  So let's skip it
+	 */
+	if (m->handler.pfds[pos].events == 0)
+		m->handler.pfds[pos].fd = -1;
 
 	if (!thread) {
 		if ((actual_state & (POLLHUP|POLLIN)) != POLLHUP)
