@@ -1,28 +1,17 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Logging of zebra
  * Copyright (C) 1997, 1998, 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #define FRR_DEFINE_DESC_TABLE
 
 #include <zebra.h>
 
+#ifdef HAVE_GLIBC_BACKTRACE
+#include <execinfo.h>
+#endif /* HAVE_GLIBC_BACKTRACE */
+
 #include "zclient.h"
 #include "log.h"
 #include "memory.h"
@@ -155,7 +144,7 @@ void zlog_signal(int signo, const char *action, void *siginfo_v,
 
 	fb.pos = buf;
 
-	struct thread *tc;
+	struct event *tc;
 	tc = pthread_getspecific(thread_current);
 
 	if (!tc)
@@ -299,7 +288,7 @@ void zlog_backtrace(int priority)
 
 void zlog_thread_info(int log_level)
 {
-	struct thread *tc;
+	struct event *tc;
 	tc = pthread_getspecific(thread_current);
 
 	if (tc)
@@ -317,7 +306,7 @@ void memory_oom(size_t size, const char *name)
 	     "out of memory: failed to allocate %zu bytes for %s object",
 	     size, name);
 	zlog_backtrace(LOG_CRIT);
-	log_memstats(stderr, "log");
+	log_memstats(zlog_progname, true);
 	abort();
 }
 
@@ -337,6 +326,7 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_INTERFACE_UP),
 	DESC_ENTRY(ZEBRA_INTERFACE_DOWN),
 	DESC_ENTRY(ZEBRA_INTERFACE_SET_MASTER),
+	DESC_ENTRY(ZEBRA_INTERFACE_SET_PROTODOWN),
 	DESC_ENTRY(ZEBRA_ROUTE_ADD),
 	DESC_ENTRY(ZEBRA_ROUTE_DELETE),
 	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_OWNER),
@@ -361,16 +351,14 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_BFD_DEST_REPLAY),
 	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_ADD),
 	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_DEL),
-	DESC_ENTRY(ZEBRA_VRF_UNREGISTER),
 	DESC_ENTRY(ZEBRA_VRF_ADD),
 	DESC_ENTRY(ZEBRA_VRF_DELETE),
 	DESC_ENTRY(ZEBRA_VRF_LABEL),
-	DESC_ENTRY(ZEBRA_INTERFACE_VRF_UPDATE),
 	DESC_ENTRY(ZEBRA_BFD_CLIENT_REGISTER),
 	DESC_ENTRY(ZEBRA_BFD_CLIENT_DEREGISTER),
 	DESC_ENTRY(ZEBRA_INTERFACE_ENABLE_RADV),
 	DESC_ENTRY(ZEBRA_INTERFACE_DISABLE_RADV),
-	DESC_ENTRY(ZEBRA_NEXTHOP_LOOKUP_MRIB),
+	DESC_ENTRY(ZEBRA_NEXTHOP_LOOKUP),
 	DESC_ENTRY(ZEBRA_INTERFACE_LINK_PARAMS),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_ADD),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_DELETE),
@@ -386,10 +374,10 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_FEC_REGISTER),
 	DESC_ENTRY(ZEBRA_FEC_UNREGISTER),
 	DESC_ENTRY(ZEBRA_FEC_UPDATE),
-	DESC_ENTRY(ZEBRA_ADVERTISE_ALL_VNI),
 	DESC_ENTRY(ZEBRA_ADVERTISE_DEFAULT_GW),
 	DESC_ENTRY(ZEBRA_ADVERTISE_SVI_MACIP),
 	DESC_ENTRY(ZEBRA_ADVERTISE_SUBNET),
+	DESC_ENTRY(ZEBRA_ADVERTISE_ALL_VNI),
 	DESC_ENTRY(ZEBRA_LOCAL_ES_ADD),
 	DESC_ENTRY(ZEBRA_LOCAL_ES_DEL),
 	DESC_ENTRY(ZEBRA_REMOTE_ES_VTEP_ADD),
@@ -438,30 +426,46 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_MLAG_CLIENT_REGISTER),
 	DESC_ENTRY(ZEBRA_MLAG_CLIENT_UNREGISTER),
 	DESC_ENTRY(ZEBRA_MLAG_FORWARD_MSG),
+	DESC_ENTRY(ZEBRA_NHG_ADD),
+	DESC_ENTRY(ZEBRA_NHG_DEL),
+	DESC_ENTRY(ZEBRA_NHG_NOTIFY_OWNER),
+	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_ADD),
+	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_DEL),
+	DESC_ENTRY(ZEBRA_SRV6_LOCATOR_ADD),
+	DESC_ENTRY(ZEBRA_SRV6_LOCATOR_DELETE),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_LOCATOR_CHUNK),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_RELEASE_LOCATOR_CHUNK),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_LOCATOR),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_SRV6_SID),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_RELEASE_SRV6_SID),
 	DESC_ENTRY(ZEBRA_ERROR),
 	DESC_ENTRY(ZEBRA_CLIENT_CAPABILITIES),
 	DESC_ENTRY(ZEBRA_OPAQUE_MESSAGE),
 	DESC_ENTRY(ZEBRA_OPAQUE_REGISTER),
 	DESC_ENTRY(ZEBRA_OPAQUE_UNREGISTER),
 	DESC_ENTRY(ZEBRA_NEIGH_DISCOVER),
-	DESC_ENTRY(ZEBRA_NHG_ADD),
-	DESC_ENTRY(ZEBRA_NHG_DEL),
-	DESC_ENTRY(ZEBRA_NHG_NOTIFY_OWNER),
 	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_REQUEST),
 	DESC_ENTRY(ZEBRA_CLIENT_CLOSE_NOTIFY),
-	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_ADD),
-	DESC_ENTRY(ZEBRA_EVPN_REMOTE_NH_DEL),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_ADDED),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REMOVED),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_GET),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REGISTER),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_UNREGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_ADDED),
+	DESC_ENTRY(ZEBRA_NEIGH_REMOVED),
+	DESC_ENTRY(ZEBRA_NEIGH_GET),
+	DESC_ENTRY(ZEBRA_NEIGH_REGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_UNREGISTER),
 	DESC_ENTRY(ZEBRA_NEIGH_IP_ADD),
 	DESC_ENTRY(ZEBRA_NEIGH_IP_DEL),
 	DESC_ENTRY(ZEBRA_CONFIGURE_ARP),
 	DESC_ENTRY(ZEBRA_GRE_GET),
 	DESC_ENTRY(ZEBRA_GRE_UPDATE),
-	DESC_ENTRY(ZEBRA_GRE_SOURCE_SET)};
+	DESC_ENTRY(ZEBRA_GRE_SOURCE_SET),
+	DESC_ENTRY(ZEBRA_TC_QDISC_INSTALL),
+	DESC_ENTRY(ZEBRA_TC_QDISC_UNINSTALL),
+	DESC_ENTRY(ZEBRA_TC_CLASS_ADD),
+	DESC_ENTRY(ZEBRA_TC_CLASS_DELETE),
+	DESC_ENTRY(ZEBRA_TC_FILTER_ADD),
+	DESC_ENTRY(ZEBRA_TC_FILTER_DELETE),
+	DESC_ENTRY(ZEBRA_OPAQUE_NOTIFY),
+	DESC_ENTRY(ZEBRA_SRV6_SID_NOTIFY)
+};
 #undef DESC_ENTRY
 
 static const struct zebra_desc_table unknown = {0, "unknown", '?'};
@@ -510,6 +514,26 @@ const char *zserv_command_string(unsigned int command)
 	return command_types[command].string;
 }
 
+#define DESC_ENTRY(T) [(T)] = {(T), (#T), '\0'}
+static const struct zebra_desc_table gr_client_cap_types[] = {
+	DESC_ENTRY(ZEBRA_CLIENT_GR_CAPABILITIES),
+	DESC_ENTRY(ZEBRA_CLIENT_ROUTE_UPDATE_COMPLETE),
+	DESC_ENTRY(ZEBRA_CLIENT_ROUTE_UPDATE_PENDING),
+	DESC_ENTRY(ZEBRA_CLIENT_GR_DISABLE),
+	DESC_ENTRY(ZEBRA_CLIENT_RIB_STALE_TIME),
+};
+#undef DESC_ENTRY
+
+const char *zserv_gr_client_cap_string(uint32_t zcc)
+{
+	if (zcc >= array_size(gr_client_cap_types)) {
+		flog_err(EC_LIB_DEVELOPMENT, "unknown zserv command type: %u",
+			 zcc);
+		return unknown.string;
+	}
+	return gr_client_cap_types[zcc].string;
+}
+
 int proto_name2num(const char *s)
 {
 	unsigned i;
@@ -530,6 +554,8 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_KERNEL;
 		else if (strmatch(s, "connected"))
 			return ZEBRA_ROUTE_CONNECT;
+		else if (strmatch(s, "local"))
+			return ZEBRA_ROUTE_LOCAL;
 		else if (strmatch(s, "static"))
 			return ZEBRA_ROUTE_STATIC;
 		else if (strmatch(s, "rip"))
@@ -556,12 +582,16 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_SHARP;
 		else if (strmatch(s, "openfabric"))
 			return ZEBRA_ROUTE_OPENFABRIC;
+		else if (strmatch(s, "table-direct"))
+			return ZEBRA_ROUTE_TABLE_DIRECT;
 	}
 	if (afi == AFI_IP6) {
 		if (strmatch(s, "kernel"))
 			return ZEBRA_ROUTE_KERNEL;
 		else if (strmatch(s, "connected"))
 			return ZEBRA_ROUTE_CONNECT;
+		else if (strmatch(s, "local"))
+			return ZEBRA_ROUTE_LOCAL;
 		else if (strmatch(s, "static"))
 			return ZEBRA_ROUTE_STATIC;
 		else if (strmatch(s, "ripng"))
@@ -586,6 +616,8 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_SHARP;
 		else if (strmatch(s, "openfabric"))
 			return ZEBRA_ROUTE_OPENFABRIC;
+		else if (strmatch(s, "table-direct"))
+			return ZEBRA_ROUTE_TABLE_DIRECT;
 	}
 	return -1;
 }
