@@ -22,13 +22,19 @@
 #include "northbound_db.h"
 #include "lib/northbound_cli_clippy.c"
 
-struct debug nb_dbg_cbs_config = {0, "Northbound callbacks: configuration"};
-struct debug nb_dbg_cbs_state = {0, "Northbound callbacks: state"};
-struct debug nb_dbg_cbs_rpc = {0, "Northbound callbacks: RPCs"};
-struct debug nb_dbg_cbs_notify = {0, "Northbound callbacks: notifications"};
-struct debug nb_dbg_notif = {0, "Northbound notifications"};
-struct debug nb_dbg_events = {0, "Northbound events"};
-struct debug nb_dbg_libyang = {0, "libyang debugging"};
+struct debug nb_dbg_cbs_config = { 0, "debug northbound callbacks configuration",
+				   "Northbound callbacks: configuration" };
+struct debug nb_dbg_cbs_state = { 0, "debug northbound callbacks state",
+				  "Northbound callbacks: state" };
+struct debug nb_dbg_cbs_rpc = { 0, "debug northbound callbacks rpc",
+				"Northbound callbacks: RPCs" };
+struct debug nb_dbg_cbs_notify = { 0, "debug northbound callbacks notify",
+				   "Northbound callbacks: notifications" };
+struct debug nb_dbg_notif = { 0, "debug northbound notifications",
+			      "Northbound notifications" };
+struct debug nb_dbg_events = { 0, "debug northbound events",
+			       "Northbound events" };
+struct debug nb_dbg_libyang = { 0, "debug northbound libyang", "libyang" };
 
 struct nb_config *vty_shared_candidate_config;
 static struct event_loop *master;
@@ -77,6 +83,7 @@ static int nb_cli_classic_commit(struct vty *vty)
 static void nb_cli_pending_commit_clear(struct vty *vty)
 {
 	vty->pending_commit = 0;
+	vty->buffer_cmd_count = 0;
 	XFREE(MTYPE_TMP, vty->pending_cmds_buf);
 	vty->pending_cmds_buflen = 0;
 	vty->pending_cmds_bufpos = 0;
@@ -96,12 +103,19 @@ int nb_cli_pending_commit_check(struct vty *vty)
 
 static int nb_cli_schedule_command(struct vty *vty)
 {
-	/* Append command to dynamically sized buffer of scheduled commands. */
+	/* Append command to dynamically sized buffer of scheduled commands.
+	 * vty->buf -Incoming config
+	 * vty->pending_cmds_buf - Pending buffer where incoming configs are
+	 *                         accumulated for later processing
+	 * vty->pending_cmds_bufpos - length of the pending buffer
+	 *
+	 */
 	if (!vty->pending_cmds_buf) {
 		vty->pending_cmds_buflen = 4096;
 		vty->pending_cmds_buf =
 			XCALLOC(MTYPE_TMP, vty->pending_cmds_buflen);
 	}
+
 	if ((strlen(vty->buf) + 3)
 	    > (vty->pending_cmds_buflen - vty->pending_cmds_bufpos)) {
 		vty->pending_cmds_buflen *= 2;
@@ -115,6 +129,9 @@ static int nb_cli_schedule_command(struct vty *vty)
 
 	/* Schedule the commit operation. */
 	vty->pending_commit = 1;
+	vty->buffer_cmd_count++;
+	if (vty->buffer_cmd_count == NB_CMD_BATCH_SIZE)
+		nb_cli_pending_commit_check(vty);
 
 	return CMD_SUCCESS;
 }
@@ -275,10 +292,31 @@ int nb_cli_apply_changes_clear_pending(struct vty *vty,
 	return nb_cli_apply_changes_internal(vty, xpath_base_abs, true);
 }
 
-int nb_cli_rpc(struct vty *vty, const char *xpath, struct list *input,
-	       struct list *output)
+int nb_cli_rpc_enqueue(struct vty *vty, const char *xpath, const char *value)
+{
+	struct nb_cfg_change *param;
+
+	if (vty->num_rpc_params == VTY_MAXCFGCHANGES) {
+		/* Not expected to happen. */
+		vty_out(vty,
+			"%% Exceeded the maximum number of params (%u) for a single command\n\n",
+			VTY_MAXCFGCHANGES);
+		return CMD_WARNING;
+	}
+
+	param = &vty->rpc_params[vty->num_rpc_params++];
+	strlcpy(param->xpath, xpath, sizeof(param->xpath));
+	param->value = value;
+
+	return CMD_SUCCESS;
+}
+
+int nb_cli_rpc(struct vty *vty, const char *xpath, struct lyd_node **output_p)
 {
 	struct nb_node *nb_node;
+	struct lyd_node *input = NULL;
+	struct lyd_node *output = NULL;
+	LY_ERR err;
 	int ret;
 	char errmsg[BUFSIZ] = {0};
 
@@ -289,12 +327,62 @@ int nb_cli_rpc(struct vty *vty, const char *xpath, struct list *input,
 		return CMD_WARNING;
 	}
 
+	/* create input tree */
+	err = lyd_new_path2(NULL, ly_native_ctx, xpath, NULL, 0, 0, 0, NULL,
+			    &input);
+	assert(err == LY_SUCCESS);
+
+	for (size_t i = 0; i < vty->num_rpc_params; i++) {
+		err = lyd_new_path(input, ly_native_ctx,
+				   vty->rpc_params[i].xpath,
+				   vty->rpc_params[i].value, 0, NULL);
+		assert(err == LY_SUCCESS);
+	}
+
+	if (vty_mgmt_fe_enabled()) {
+		char *data = NULL;
+
+		err = lyd_print_mem(&data, input, LYD_JSON, LYD_PRINT_SHRINK);
+		assert(err == LY_SUCCESS);
+
+		ret = vty_mgmt_send_rpc_req(vty, LYD_JSON, xpath, data);
+
+		free(data);
+		lyd_free_all(input);
+
+		if (ret < 0)
+			return CMD_WARNING;
+		return CMD_SUCCESS;
+	}
+
+	/* validate input tree to create implicit defaults */
+	err = lyd_validate_op(input, NULL, LYD_TYPE_RPC_YANG, NULL);
+	assert(err == LY_SUCCESS);
+
+	/* create output tree root for population in the callback */
+	err = lyd_new_path2(NULL, ly_native_ctx, xpath, NULL, 0, 0, 0, NULL,
+			    &output);
+	assert(err == LY_SUCCESS);
+
 	ret = nb_callback_rpc(nb_node, xpath, input, output, errmsg,
 			      sizeof(errmsg));
+
+	/* validate output tree to create implicit defaults */
+	err = lyd_validate_op(output, NULL, LYD_TYPE_REPLY_YANG, NULL);
+	assert(err == LY_SUCCESS);
+
+	lyd_free_all(input);
+	vty->num_rpc_params = 0;
+
 	switch (ret) {
 	case NB_OK:
+		if (output_p)
+			*output_p = output;
+		else
+			lyd_free_all(output);
 		return CMD_SUCCESS;
 	default:
+		lyd_free_all(output);
 		if (strlen(errmsg))
 			vty_show_nb_errors(vty, ret, errmsg);
 		return CMD_WARNING;
@@ -1309,7 +1397,7 @@ static int nb_cli_show_transactions(struct vty *vty)
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No configuration transactions to display.\n\n");
 
@@ -1590,7 +1678,7 @@ DEFPY (show_yang_module,
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No YANG modules to display.\n\n");
 
@@ -1700,7 +1788,7 @@ DEFPY (show_yang_module_translator,
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No YANG module translators to display.\n\n");
 
@@ -1771,37 +1859,6 @@ DEFPY (rollback_config,
 }
 
 /* Debug CLI commands. */
-static struct debug *nb_debugs[] = {
-	&nb_dbg_cbs_config, &nb_dbg_cbs_state, &nb_dbg_cbs_rpc,
-	&nb_dbg_cbs_notify, &nb_dbg_notif,     &nb_dbg_events,
-	&nb_dbg_libyang,
-};
-
-static const char *const nb_debugs_conflines[] = {
-	"debug northbound callbacks configuration",
-	"debug northbound callbacks state",
-	"debug northbound callbacks rpc",
-	"debug northbound callbacks notify",
-	"debug northbound notifications",
-	"debug northbound events",
-	"debug northbound libyang",
-};
-
-DEFINE_HOOK(nb_client_debug_set_all, (uint32_t flags, bool set), (flags, set));
-
-static void nb_debug_set_all(uint32_t flags, bool set)
-{
-	for (unsigned int i = 0; i < array_size(nb_debugs); i++) {
-		DEBUG_FLAGS_SET(nb_debugs[i], flags, set);
-
-		/* If all modes have been turned off, don't preserve options. */
-		if (!DEBUG_MODE_CHECK(nb_debugs[i], DEBUG_MODE_ALL))
-			DEBUG_CLEAR(nb_debugs[i]);
-	}
-
-	hook_call(nb_client_debug_set_all, flags, set);
-}
-
 DEFPY (debug_nb,
        debug_nb_cmd,
        "[no] debug northbound\
@@ -1824,8 +1881,13 @@ DEFPY (debug_nb,
        "libyang debugging\n")
 {
 	uint32_t mode = DEBUG_NODE2MODE(vty->node);
+	bool all = false;
 
-	if (cbs) {
+	/* no specific debug --> act on all of them */
+	if (strmatch(argv[argc - 1]->text, "northbound"))
+		all = true;
+
+	if (cbs || all) {
 		bool none = (!cbs_cfg && !cbs_state && !cbs_rpc && !cbs_notify);
 
 		if (none || cbs_cfg)
@@ -1837,45 +1899,18 @@ DEFPY (debug_nb,
 		if (none || cbs_notify)
 			DEBUG_MODE_SET(&nb_dbg_cbs_notify, mode, !no);
 	}
-	if (notifications)
+	if (notifications || all)
 		DEBUG_MODE_SET(&nb_dbg_notif, mode, !no);
-	if (events)
+	if (events || all)
 		DEBUG_MODE_SET(&nb_dbg_events, mode, !no);
-	if (libyang) {
+	if (libyang || all) {
 		DEBUG_MODE_SET(&nb_dbg_libyang, mode, !no);
 		yang_debugging_set(!no);
 	}
 
-	/* no specific debug --> act on all of them */
-	if (strmatch(argv[argc - 1]->text, "northbound")) {
-		nb_debug_set_all(mode, !no);
-		yang_debugging_set(!no);
-	}
-
 	return CMD_SUCCESS;
 }
 
-DEFINE_HOOK(nb_client_debug_config_write, (struct vty *vty), (vty));
-
-static int nb_debug_config_write(struct vty *vty)
-{
-	for (unsigned int i = 0; i < array_size(nb_debugs); i++)
-		if (DEBUG_MODE_CHECK(nb_debugs[i], DEBUG_MODE_CONF))
-			vty_out(vty, "%s\n", nb_debugs_conflines[i]);
-
-	hook_call(nb_client_debug_config_write, vty);
-
-	return 1;
-}
-
-static struct debug_callbacks nb_dbg_cbs = {.debug_set_all = nb_debug_set_all};
-static struct cmd_node nb_debug_node = {
-	.name = "northbound debug",
-	.node = NORTHBOUND_DEBUG_NODE,
-	.prompt = "",
-	.config_write = nb_debug_config_write,
-};
-
 void nb_cli_install_default(int node)
 {
 	_install_element(node, &show_config_candidate_section_cmd);
@@ -1936,9 +1971,14 @@ void nb_cli_init(struct event_loop *tm)
 	/* Initialize the shared candidate configuration. */
 	vty_shared_candidate_config = nb_config_new(NULL);
 
-	debug_init(&nb_dbg_cbs);
+	debug_install(&nb_dbg_cbs_config);
+	debug_install(&nb_dbg_cbs_state);
+	debug_install(&nb_dbg_cbs_rpc);
+	debug_install(&nb_dbg_cbs_notify);
+	debug_install(&nb_dbg_notif);
+	debug_install(&nb_dbg_events);
+	debug_install(&nb_dbg_libyang);
 
-	install_node(&nb_debug_node);
 	install_element(ENABLE_NODE, &debug_nb_cmd);
 	install_element(CONFIG_NODE, &debug_nb_cmd);
 
