@@ -875,6 +875,28 @@ void route_map_walk_update_list(void (*route_map_update_fn)(char *name))
 	}
 }
 
+static const char *route_map_action_reason2str(enum route_map_action_reason reason)
+{
+	switch (reason) {
+	case route_map_action_none:
+		return "none";
+	case route_map_action_map_null:
+		return "route-map is null";
+	case route_map_action_no_index:
+		return "no index";
+	case route_map_action_next_deny:
+		return "next statement is deny";
+	case route_map_action_exit:
+		return "exit policy";
+	case route_map_action_goto_null:
+		return "goto index is null";
+	case route_map_action_index_deny:
+		return "deny index";
+	}
+
+	return "Invalid reason";
+}
+
 /* Return route map's type string. */
 static const char *route_map_type_str(enum route_map_type type)
 {
@@ -941,11 +963,12 @@ static void vty_show_route_map_entry(struct vty *vty, struct route_map *map,
 		json_object_boolean_add(json_rmap, "processedChange",
 					map->to_be_processed);
 		json_object_object_add(json_rmap, "rules", json_rules);
+		json_object_int_add(json_rmap, "cpuTimeMS", map->cputime / 1000);
 	} else {
 		vty_out(vty,
 			"route-map: %s Invoked: %" PRIu64
-			" Optimization: %s Processed Change: %s\n",
-			map->name, map->applied - map->applied_clear,
+			" (%zu milliseconds total) Optimization: %s Processed Change: %s\n",
+			map->name, map->applied - map->applied_clear, map->cputime / 1000,
 			map->optimization_disabled ? "disabled" : "enabled",
 			map->to_be_processed ? "true" : "false");
 	}
@@ -967,6 +990,7 @@ static void vty_show_route_map_entry(struct vty *vty, struct route_map *map,
 			json_object_int_add(json_rule, "invoked",
 					    index->applied
 						    - index->applied_clear);
+			json_object_int_add(json_rule, "cpuTimeMS", index->cputime / 1000);
 
 			/* Description */
 			if (index->description)
@@ -1018,9 +1042,10 @@ static void vty_show_route_map_entry(struct vty *vty, struct route_map *map,
 				json_object_string_add(json_rule, "action",
 						       action);
 		} else {
-			vty_out(vty, " %s, sequence %d Invoked %" PRIu64 "\n",
+			vty_out(vty,
+				" %s, sequence %d Invoked %" PRIu64 " (%zu milliseconds total)\n",
 				route_map_type_str(index->type), index->pref,
-				index->applied - index->applied_clear);
+				index->applied - index->applied_clear, index->cputime / 1000);
 
 			/* Description */
 			if (index->description)
@@ -2548,6 +2573,10 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 	struct route_map_index *index = NULL;
 	struct route_map_rule *set = NULL;
 	bool skip_match_clause = false;
+	RUSAGE_T mbefore, mafter;
+	RUSAGE_T ibefore, iafter;
+	unsigned long cputime;
+	enum route_map_action_reason reason = route_map_action_none;
 
 	if (recursion > RMAP_RECURSION_LIMIT) {
 		if (map)
@@ -2565,11 +2594,15 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 		if (map)
 			map->applied++;
 		ret = RMAP_DENYMATCH;
+		reason = route_map_action_map_null;
 		goto route_map_apply_end;
 	}
 
 	map->applied++;
 
+	GETRUSAGE(&mbefore);
+	ibefore = mbefore;
+
 	if (prefix->family == AF_EVPN) {
 		index = map->head;
 	} else {
@@ -2580,6 +2613,12 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 
 	if (index) {
 		index->applied++;
+
+		GETRUSAGE(&iafter);
+		event_consumed_time(&iafter, &ibefore, &cputime);
+		index->cputime += cputime;
+		ibefore = iafter;
+
 		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
 			zlog_debug(
 				"Best match route-map: %s, sequence: %d for pfx: %pFX, result: %s",
@@ -2599,6 +2638,7 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 			ret = RMAP_PERMITMATCH;
 		else
 			ret = RMAP_DENYMATCH;
+		reason = route_map_action_no_index;
 		goto route_map_apply_end;
 	}
 
@@ -2686,12 +2726,15 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 					}
 
 					/* If nextrm returned 'deny', finish. */
-					if (ret == RMAP_DENYMATCH)
+					if (ret == RMAP_DENYMATCH) {
+						reason = route_map_action_next_deny;
 						goto route_map_apply_end;
+					}
 				}
 
 				switch (index->exitpolicy) {
 				case RMAP_EXIT:
+					reason = route_map_action_exit;
 					goto route_map_apply_end;
 				case RMAP_NEXT:
 					continue;
@@ -2707,6 +2750,7 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 					}
 					if (next == NULL) {
 						/* No clauses match! */
+						reason = route_map_action_goto_null;
 						goto route_map_apply_end;
 					}
 				}
@@ -2715,16 +2759,21 @@ route_map_result_t route_map_apply_ext(struct route_map *map,
 			/* 'deny' */
 			{
 				ret = RMAP_DENYMATCH;
+				reason = route_map_action_index_deny;
 				goto route_map_apply_end;
 			}
 		}
+		GETRUSAGE(&iafter);
+		event_consumed_time(&iafter, &ibefore, &cputime);
+		index->cputime += cputime;
+		ibefore = iafter;
 	}
 
 route_map_apply_end:
 	if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP)))
-		zlog_debug("Route-map: %s, prefix: %pFX, result: %s",
-			   (map ? map->name : "null"), prefix,
-			   route_map_result_str(ret));
+		zlog_debug("Route-map: %s, prefix: %pFX, result: %s, reason: %s",
+			   (map ? map->name : "null"), prefix, route_map_result_str(ret),
+			   route_map_action_reason2str(reason));
 
 	if (pref) {
 		if (index != NULL && ret == RMAP_PERMITMATCH)
@@ -2733,6 +2782,13 @@ route_map_apply_end:
 			*pref = 65536;
 	}
 
+	if (map) {
+		GETRUSAGE(&mbefore);
+		GETRUSAGE(&mafter);
+		event_consumed_time(&mafter, &mbefore, &cputime);
+		map->cputime += cputime;
+	}
+
 	return (ret);
 }
 
@@ -3090,8 +3146,11 @@ static void clear_route_map_helper(struct route_map *map)
 	struct route_map_index *index;
 
 	map->applied_clear = map->applied;
-	for (index = map->head; index; index = index->next)
+	map->cputime = 0;
+	for (index = map->head; index; index = index->next) {
 		index->applied_clear = index->applied;
+		index->cputime = 0;
+	}
 }
 
 DEFPY (rmap_clear_counters,
