@@ -71,12 +71,19 @@ int snprintf_seg6_segs(char *str,
 	return strlen(str);
 }
 
+void srv6_sid_structure2json(const struct seg6local_context *ctx, json_object *json)
+{
+	json_object_int_add(json, "blockLen", ctx->block_len);
+	json_object_int_add(json, "nodeLen", ctx->node_len);
+	json_object_int_add(json, "funcLen", ctx->function_len);
+	json_object_int_add(json, "argLen", ctx->argument_len);
+}
+
 void seg6local_context2json(const struct seg6local_context *ctx,
 			    uint32_t action, json_object *json)
 {
 	switch (action) {
 	case ZEBRA_SEG6_LOCAL_ACTION_END:
-		json_object_boolean_add(json, "USP", true);
 		return;
 	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX6:
@@ -116,7 +123,7 @@ const char *seg6local_context2str(char *str, size_t size,
 	switch (action) {
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END:
-		snprintf(str, size, "USP");
+		snprintf(str, size, "-");
 		return str;
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
