@@ -131,7 +131,7 @@ typedef enum {
 	ZEBRA_BFD_CLIENT_DEREGISTER,
 	ZEBRA_INTERFACE_ENABLE_RADV,
 	ZEBRA_INTERFACE_DISABLE_RADV,
-	ZEBRA_NEXTHOP_LOOKUP_MRIB,
+	ZEBRA_NEXTHOP_LOOKUP,
 	ZEBRA_INTERFACE_LINK_PARAMS,
 	ZEBRA_MPLS_LABELS_ADD,
 	ZEBRA_MPLS_LABELS_DELETE,
@@ -251,21 +251,16 @@ enum zebra_error_types {
 
 static inline const char *zebra_error_type2str(enum zebra_error_types type)
 {
-	const char *ret = "UNKNOWN";
-
 	switch (type) {
 	case ZEBRA_UNKNOWN_ERROR:
-		ret = "ZEBRA_UNKNOWN_ERROR";
-		break;
+		return "ZEBRA_UNKNOWN_ERROR";
 	case ZEBRA_NO_VRF:
-		ret = "ZEBRA_NO_VRF";
-		break;
+		return "ZEBRA_NO_VRF";
 	case ZEBRA_INVALID_MSG_TYPE:
-		ret = "ZEBRA_INVALID_MSG_TYPE";
-		break;
+		return "ZEBRA_INVALID_MSG_TYPE";
 	}
 
-	return ret;
+	return "UNKNOWN";
 }
 
 struct redist_proto {
@@ -273,6 +268,21 @@ struct redist_proto {
 	struct list *instances;
 };
 
+/**
+ * Redistribute table direct instance data structure: keeps the VRF
+ * that subscribed to the table ID.
+ *
+ * **NOTE**
+ * `table_id` is an integer because that is what the netlink interface
+ * uses for route attribute RTA_TABLE (32bit int), however the whole
+ * zclient API uses `unsigned short` (and CLI commands) so it will be
+ * limited to the range 1 to 65535.
+ */
+struct redist_table_direct {
+	vrf_id_t vrf_id;
+	int table_id;
+};
+
 struct zclient_capabilities {
 	uint32_t ecmp;
 	bool mpls_enabled;
@@ -780,69 +790,51 @@ enum zclient_send_status {
 static inline const char *
 zapi_nhg_notify_owner2str(enum zapi_nhg_notify_owner note)
 {
-	const char *ret = "UNKNOWN";
-
 	switch (note) {
 	case ZAPI_NHG_FAIL_INSTALL:
-		ret = "ZAPI_NHG_FAIL_INSTALL";
-		break;
+		return "ZAPI_NHG_FAIL_INSTALL";
 	case ZAPI_NHG_INSTALLED:
-		ret = "ZAPI_NHG_INSTALLED";
-		break;
+		return "ZAPI_NHG_INSTALLED";
 	case ZAPI_NHG_REMOVE_FAIL:
-		ret = "ZAPI_NHG_REMOVE_FAIL";
-		break;
+		return "ZAPI_NHG_REMOVE_FAIL";
 	case ZAPI_NHG_REMOVED:
-		ret = "ZAPI_NHG_REMOVED";
-		break;
+		return "ZAPI_NHG_REMOVED";
 	}
 
-	return ret;
+	return "UNKNOWN";
 }
 
 static inline const char *
 zapi_rule_notify_owner2str(enum zapi_rule_notify_owner note)
 {
-	const char *ret = "UNKNOWN";
-
 	switch (note) {
 	case ZAPI_RULE_FAIL_INSTALL:
-		ret = "ZAPI_RULE_FAIL_INSTALL";
-		break;
+		return "ZAPI_RULE_FAIL_INSTALL";
 	case ZAPI_RULE_INSTALLED:
-		ret = "ZAPI_RULE_INSTALLED";
-		break;
+		return "ZAPI_RULE_INSTALLED";
 	case ZAPI_RULE_FAIL_REMOVE:
-		ret = "ZAPI_RULE_FAIL_REMOVE";
-		break;
+		return "ZAPI_RULE_FAIL_REMOVE";
 	case ZAPI_RULE_REMOVED:
-		ret = "ZAPI_RULE_REMOVED";
-		break;
+		return "ZAPI_RULE_REMOVED";
 	}
 
-	return ret;
+	return "UNKNOWN";
 }
 
 static inline const char *zapi_srv6_sid_notify2str(enum zapi_srv6_sid_notify note)
 {
-	const char *ret = "UNKNOWN";
-
 	switch (note) {
 	case ZAPI_SRV6_SID_FAIL_ALLOC:
-		ret = "ZAPI_SRV6_SID_FAIL_ALLOC";
-		break;
+		return "ZAPI_SRV6_SID_FAIL_ALLOC";
 	case ZAPI_SRV6_SID_ALLOCATED:
-		ret = "ZAPI_SRV6_SID_ALLOCATED";
-		break;
+		return "ZAPI_SRV6_SID_ALLOCATED";
 	case ZAPI_SRV6_SID_FAIL_RELEASE:
-		ret = "ZAPI_SRV6_SID_FAIL_RELEASE";
-		break;
+		return "ZAPI_SRV6_SID_FAIL_RELEASE";
 	case ZAPI_SRV6_SID_RELEASED:
-		ret = "ZAPI_SRV6_SID_RELEASED";
-		break;
+		return "ZAPI_SRV6_SID_RELEASED";
 	}
 
-	return ret;
+	return "UNKNOWN";
 }
 
 /* Zebra MAC types */
@@ -947,6 +939,15 @@ extern void redist_add_instance(struct redist_proto *, unsigned short);
 extern void redist_del_instance(struct redist_proto *, unsigned short);
 extern void redist_del_all_instances(struct redist_proto *red);
 
+extern struct redist_table_direct *
+redist_lookup_table_direct(const struct redist_proto *red, const struct redist_table_direct *table);
+extern bool redist_table_direct_has_id(const struct redist_proto *red, int table_id);
+extern void redist_add_table_direct(struct redist_proto *red,
+				    const struct redist_table_direct *table);
+extern void redist_del_table_direct(struct redist_proto *red,
+				    const struct redist_table_direct *table);
+
+
 /*
  * Send to zebra that the specified vrf is using label to resolve
  * itself for L3VPN's.  Repeated calls of this function with
@@ -1167,6 +1168,9 @@ bool zapi_route_notify_decode(struct stream *s, struct prefix *p,
 			      uint32_t *tableid,
 			      enum zapi_route_notify_owner *note,
 			      afi_t *afi, safi_t *safi);
+bool zapi_route_notify_decode_srcdest(struct stream *s, struct prefix *p, struct prefix *src_p,
+				      uint32_t *tableid, enum zapi_route_notify_owner *note,
+				      afi_t *afi, safi_t *safi);
 bool zapi_rule_notify_decode(struct stream *s, uint32_t *seqno,
 			     uint32_t *priority, uint32_t *unique, char *ifname,
 			     enum zapi_rule_notify_owner *note);
