@@ -159,6 +159,12 @@ const struct frr_yang_module_info ietf_netconf_with_defaults_info = {
  * clients into mgmtd. The modules are used by libyang in order to support
  * parsing binary data returns from the backend.
  */
+const struct frr_yang_module_info frr_backend_client_info = {
+	.name = "frr-backend",
+	.ignore_cfg_cbs = true,
+	.nodes = { { .xpath = NULL } },
+};
+
 const struct frr_yang_module_info zebra_route_map_info = {
 	.name = "frr-zebra-route-map",
 	.ignore_cfg_cbs = true,
@@ -183,6 +189,7 @@ static const struct frr_yang_module_info *const mgmt_yang_modules[] = {
 	/*
 	 * YANG module info used by backend clients get added here.
 	 */
+	&frr_backend_client_info,
 
 	&frr_zebra_cli_info,
 	&zebra_route_map_info,
@@ -231,10 +238,9 @@ int main(int argc, char **argv)
 	int buffer_size = MGMTD_SOCKET_BUF_SIZE;
 
 	frr_preinit(&mgmtd_di, argc, argv);
-	frr_opt_add(
-		"s:n" DEPRECATED_OPTIONS, longopts,
-		"  -s, --socket_size  Set MGMTD peer socket send buffer size\n"
-		"  -n, --vrfwnetns    Use NetNS as VRF backend\n");
+	frr_opt_add("s:n" DEPRECATED_OPTIONS, longopts,
+		    "  -s, --socket_size  Set MGMTD peer socket send buffer size\n"
+		    "  -n, --vrfwnetns    Use NetNS as VRF backend (deprecated, use -w)\n");
 
 	/* Command line argument treatment. */
 	while (1) {
@@ -257,6 +263,8 @@ int main(int argc, char **argv)
 			buffer_size = atoi(optarg);
 			break;
 		case 'n':
+			fprintf(stderr,
+				"The -n option is deprecated, please use global -w option instead.\n");
 			vrf_configure_backend(VRF_BACKEND_NETNS);
 			break;
 		default:
