@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIMv6 MLD querier
  * Copyright (C) 2021-2022  David Lamparter for NetDEF, Inc.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -26,13 +13,14 @@
  */
 
 #include <zebra.h>
+#include <netinet/icmp6.h>
 #include <netinet/ip6.h>
 
 #include "lib/memory.h"
 #include "lib/jhash.h"
 #include "lib/prefix.h"
 #include "lib/checksum.h"
-#include "lib/thread.h"
+#include "lib/frrevent.h"
 #include "termtable.h"
 
 #include "pimd/pim6_mld.h"
@@ -58,7 +46,7 @@ DEFINE_MTYPE_STATIC(PIMD, GM_SG, "MLD (S,G)");
 DEFINE_MTYPE_STATIC(PIMD, GM_GRP_PENDING, "MLD group query state");
 DEFINE_MTYPE_STATIC(PIMD, GM_GSQ_PENDING, "MLD group/source query aggregate");
 
-static void gm_t_query(struct thread *t);
+static void gm_t_query(struct event *t);
 static void gm_trigger_specific(struct gm_sg *sg);
 static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 			      struct timeval expire_wait);
@@ -74,7 +62,6 @@ static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 		sg->iface->ifp->name, &sg->sgaddr
 
 /* clang-format off */
-#if PIM_IPV == 6
 static const pim_addr gm_all_hosts = {
 	.s6_addr = {
 		0xff, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -96,13 +83,6 @@ static const pim_addr gm_dummy_untracked = {
 		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	},
 };
-#else
-/* 224.0.0.1 */
-static const pim_addr gm_all_hosts = { .s_addr = htonl(0xe0000001), };
-/* 224.0.0.22 */
-static const pim_addr gm_all_routers = { .s_addr = htonl(0xe0000016), };
-static const pim_addr gm_dummy_untracked = { .s_addr = 0xffffffff, };
-#endif
 /* clang-format on */
 
 #define IPV6_MULTICAST_SCOPE_LINK 2
@@ -112,7 +92,7 @@ static inline uint8_t in6_multicast_scope(const pim_addr *addr)
 	return addr->s6_addr[1] & 0xf;
 }
 
-static inline bool in6_multicast_nofwd(const pim_addr *addr)
+bool in6_multicast_nofwd(const pim_addr *addr)
 {
 	return in6_multicast_scope(addr) <= IPV6_MULTICAST_SCOPE_LINK;
 }
@@ -195,13 +175,11 @@ DECLARE_HASH(gm_gsq_pends, struct gm_gsq_pending, itm, gm_gsq_pending_cmp,
  * interface -> (S,G)
  */
 
-static int gm_sg_cmp(const struct gm_sg *a, const struct gm_sg *b)
+int gm_sg_cmp(const struct gm_sg *a, const struct gm_sg *b)
 {
 	return pim_sgaddr_cmp(a->sgaddr, b->sgaddr);
 }
 
-DECLARE_RBTREE_UNIQ(gm_sgs, struct gm_sg, itm, gm_sg_cmp);
-
 static struct gm_sg *gm_sg_find(struct gm_if *gm_ifp, pim_addr grp,
 				pim_addr src)
 {
@@ -341,8 +319,11 @@ static void gm_expiry_calc(struct gm_query_timers *timers)
 
 static void gm_sg_free(struct gm_sg *sg)
 {
+	if (pim_embedded_rp_is_embedded(&sg->sgaddr.grp))
+		pim_embedded_rp_delete(sg->iface->pim, &sg->sgaddr.grp);
+
 	/* t_sg_expiry is handled before this is reached */
-	THREAD_OFF(sg->t_sg_query);
+	EVENT_OFF(sg->t_sg_query);
 	gm_packet_sg_subs_fini(sg->subs_negative);
 	gm_packet_sg_subs_fini(sg->subs_positive);
 	XFREE(MTYPE_GM_SG, sg);
@@ -359,7 +340,8 @@ static const char *const gm_states[] = {
 };
 /* clang-format on */
 
-CPP_NOTICE("TODO: S,G entries in EXCLUDE (i.e. prune) unsupported");
+/* TODO: S,G entries in EXCLUDE (i.e. prune) unsupported" */
+
 /* tib_sg_gm_prune() below is an "un-join", it doesn't prune S,G when *,G is
  * joined.  Whether we actually want/need to support this is a separate
  * question - it is almost never used.  In fact this is exactly what RFC5790
@@ -369,6 +351,7 @@ CPP_NOTICE("TODO: S,G entries in EXCLUDE (i.e. prune) unsupported");
 static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 {
 	struct gm_if *gm_ifp = sg->iface;
+	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 	enum gm_sg_state prev, desired;
 	bool new_join;
 	struct gm_sg *grp = NULL;
@@ -417,10 +400,13 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 			gm_expiry_calc(&timers);
 			gm_sg_timer_start(gm_ifp, sg, timers.expire_wait);
 
-			THREAD_OFF(sg->t_sg_query);
-			sg->n_query = gm_ifp->cur_lmqc;
+			EVENT_OFF(sg->t_sg_query);
 			sg->query_sbit = false;
-			gm_trigger_specific(sg);
+			/* Trigger the specific queries only for querier. */
+			if (IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest)) {
+				sg->n_query = gm_ifp->cur_lmqc;
+				gm_trigger_specific(sg);
+			}
 		}
 	}
 	prev = sg->state;
@@ -432,6 +418,13 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 		new_join = gm_sg_state_want_join(desired);
 
 	if (new_join && !sg->tib_joined) {
+		pim_addr embedded_rp;
+
+		if (sg->iface->pim->embedded_rp.enable &&
+		    pim_embedded_rp_extract(&sg->sgaddr.grp, &embedded_rp) &&
+		    !pim_embedded_rp_filter_match(sg->iface->pim, &sg->sgaddr.grp))
+			pim_embedded_rp_new(sg->iface->pim, &sg->sgaddr.grp, &embedded_rp);
+
 		/* this will retry if join previously failed */
 		sg->tib_joined = tib_sg_gm_join(gm_ifp->pim, sg->sgaddr,
 						gm_ifp->ifp, &sg->oil);
@@ -451,6 +444,20 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 	}
 
 	if (desired == GM_SG_NOINFO) {
+		/*
+		 * If oil is still present then get ride of it or we will leak
+		 * this data structure.
+		 */
+		if (sg->oil)
+			sg->oil = pim_channel_oil_del(sg->oil, __func__);
+
+		/* multiple paths can lead to the last state going away;
+		 * t_sg_expire can still be running if we're arriving from
+		 * another path.
+		 */
+		if (has_expired)
+			EVENT_OFF(sg->t_sg_expire);
+
 		assertf((!sg->t_sg_expire &&
 			 !gm_packet_sg_subs_count(sg->subs_positive) &&
 			 !gm_packet_sg_subs_count(sg->subs_negative)),
@@ -478,6 +485,8 @@ static void gm_sg_update(struct gm_sg *sg, bool has_expired)
 
 static void gm_packet_free(struct gm_packet_state *pkt)
 {
+	assert(pkt->iface);
+
 	gm_packet_expires_del(pkt->iface->expires, pkt);
 	gm_packets_del(pkt->subscriber->packets, pkt);
 	gm_subscriber_drop(&pkt->subscriber);
@@ -588,7 +597,7 @@ static void gm_sg_expiry_cancel(struct gm_sg *sg)
 {
 	if (sg->t_sg_expire && PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_sg(sg, "alive, cancelling expiry timer"));
-	THREAD_OFF(sg->t_sg_expire);
+	EVENT_OFF(sg->t_sg_expire);
 	sg->query_sbit = true;
 }
 
@@ -602,7 +611,7 @@ static void gm_sg_expiry_cancel(struct gm_sg *sg)
  * everything else is thrown into pkt for creation of state in pass 2
  */
 static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
-			       struct mld_v2_rec_hdr *rechdr)
+			       struct mld_v2_rec_hdr *rechdr, size_t n_src)
 {
 	/* NB: pkt->subscriber can be NULL here if the subscriber was not
 	 * previously seen!
@@ -611,7 +620,6 @@ static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
 	struct gm_sg *grp;
 	struct gm_packet_sg *old_grp = NULL;
 	struct gm_packet_sg *item;
-	size_t n_src = ntohs(rechdr->n_src);
 	size_t j;
 	bool is_excl = false;
 
@@ -654,7 +662,7 @@ static void gm_handle_v2_pass1(struct gm_packet_state *pkt,
 			 */
 			gm_packet_sg_drop(old_grp);
 			gm_sg_update(grp, false);
-			CPP_NOTICE("need S,G PRUNE => NO_INFO transition here");
+/* TODO "need S,G PRUNE => NO_INFO transition here" */
 		}
 		break;
 
@@ -802,7 +810,8 @@ static void gm_handle_v2_pass2_excl(struct gm_packet_state *pkt, size_t offs)
 	gm_sg_update(sg_grp, false);
 }
 
-CPP_NOTICE("TODO: QRV/QQIC are not copied from queries to local state");
+/* TODO: QRV/QQIC are not copied from queries to local state" */
+
 /* on receiving a query, we need to update our robustness/query interval to
  * match, so we correctly process group/source specific queries after last
  * member leaves
@@ -824,13 +833,24 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 		return;
 	}
 
-	/* errors after this may at least partially process the packet */
-	gm_ifp->stats.rx_new_report++;
-
 	hdr = (struct mld_v2_report_hdr *)data;
 	data += sizeof(*hdr);
 	len -= sizeof(*hdr);
 
+	n_records = ntohs(hdr->n_records);
+	if (n_records > len / sizeof(struct mld_v2_rec_hdr)) {
+		/* note this is only an upper bound, records with source lists
+		 * are larger.  This is mostly here to make coverity happy.
+		 */
+		zlog_warn(log_pkt_src(
+			"malformed MLDv2 report (infeasible record count)"));
+		gm_ifp->stats.rx_drop_malformed++;
+		return;
+	}
+
+	/* errors after this may at least partially process the packet */
+	gm_ifp->stats.rx_new_report++;
+
 	/* can't have more *,G and S,G items than there is space for ipv6
 	 * addresses, so just use this to allocate temporary buffer
 	 */
@@ -841,8 +861,6 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 	pkt->iface = gm_ifp;
 	pkt->subscriber = gm_subscriber_findref(gm_ifp, pkt_src->sin6_addr);
 
-	n_records = ntohs(hdr->n_records);
-
 	/* validate & remove state in v2_pass1() */
 	for (i = 0; i < n_records; i++) {
 		struct mld_v2_rec_hdr *rechdr;
@@ -880,7 +898,7 @@ static void gm_handle_v2_report(struct gm_if *gm_ifp,
 		data += record_size;
 		len -= record_size;
 
-		gm_handle_v2_pass1(pkt, rechdr);
+		gm_handle_v2_pass1(pkt, rechdr, n_src);
 	}
 
 	if (!pkt->n_active) {
@@ -949,7 +967,8 @@ static void gm_handle_v1_report(struct gm_if *gm_ifp,
 
 	item = gm_packet_sg_setup(pkt, grp, true, false);
 	item->n_exclude = 0;
-	CPP_NOTICE("set v1-seen timer on grp here");
+
+/* TODO "set v1-seen timer on grp here" */
 
 	/* } */
 
@@ -1012,7 +1031,9 @@ static void gm_handle_v1_leave(struct gm_if *gm_ifp,
 		if (old_grp) {
 			gm_packet_sg_drop(old_grp);
 			gm_sg_update(grp, false);
-			CPP_NOTICE("need S,G PRUNE => NO_INFO transition here");
+
+/* TODO "need S,G PRUNE => NO_INFO transition here" */
+
 		}
 	}
 
@@ -1035,9 +1056,9 @@ static void gm_handle_v1_leave(struct gm_if *gm_ifp,
  * its own path too and won't hit this.  This is really only triggered when a
  * host straight up disappears.
  */
-static void gm_t_expire(struct thread *t)
+static void gm_t_expire(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	struct gm_packet_state *pkt;
 
 	zlog_info(log_ifp("general expiry timer"));
@@ -1054,8 +1075,8 @@ static void gm_t_expire(struct thread *t)
 					log_ifp("next general expiry in %" PRId64 "ms"),
 					remain_ms / 1000);
 
-			thread_add_timer_tv(router->master, gm_t_expire, gm_ifp,
-					    &remain, &gm_ifp->t_expire);
+			event_add_timer_tv(router->master, gm_t_expire, gm_ifp,
+					   &remain, &gm_ifp->t_expire);
 			return;
 		}
 
@@ -1109,7 +1130,7 @@ static void gm_handle_q_general(struct gm_if *gm_ifp,
 
 		gm_ifp->n_pending--;
 		if (!gm_ifp->n_pending)
-			THREAD_OFF(gm_ifp->t_expire);
+			EVENT_OFF(gm_ifp->t_expire);
 	}
 
 	/* people might be messing with their configs or something */
@@ -1125,16 +1146,16 @@ static void gm_handle_q_general(struct gm_if *gm_ifp,
 			zlog_debug(
 				log_ifp("starting general timer @ 0: %pTVMu"),
 				&pend->expiry);
-		thread_add_timer_tv(router->master, gm_t_expire, gm_ifp,
-				    &timers->expire_wait, &gm_ifp->t_expire);
+		event_add_timer_tv(router->master, gm_t_expire, gm_ifp,
+				   &timers->expire_wait, &gm_ifp->t_expire);
 	} else if (PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_ifp("appending general timer @ %u: %pTVMu"),
 			   gm_ifp->n_pending, &pend->expiry);
 }
 
-static void gm_t_sg_expire(struct thread *t)
+static void gm_t_sg_expire(struct event *t)
 {
-	struct gm_sg *sg = THREAD_ARG(t);
+	struct gm_sg *sg = EVENT_ARG(t);
 	struct gm_if *gm_ifp = sg->iface;
 	struct gm_packet_sg *item;
 
@@ -1214,15 +1235,15 @@ static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 	if (sg->t_sg_expire) {
 		struct timeval remain;
 
-		remain = thread_timer_remain(sg->t_sg_expire);
+		remain = event_timer_remain(sg->t_sg_expire);
 		if (timercmp(&remain, &expire_wait, <=))
 			return;
 
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 	}
 
-	thread_add_timer_tv(router->master, gm_t_sg_expire, sg, &expire_wait,
-			    &sg->t_sg_expire);
+	event_add_timer_tv(router->master, gm_t_sg_expire, sg, &expire_wait,
+			   &sg->t_sg_expire);
 }
 
 static void gm_handle_q_groupsrc(struct gm_if *gm_ifp,
@@ -1234,11 +1255,12 @@ static void gm_handle_q_groupsrc(struct gm_if *gm_ifp,
 
 	for (i = 0; i < n_src; i++) {
 		sg = gm_sg_find(gm_ifp, grp, srcs[i]);
+		GM_UPDATE_SG_STATE(sg);
 		gm_sg_timer_start(gm_ifp, sg, timers->expire_wait);
 	}
 }
 
-static void gm_t_grp_expire(struct thread *t)
+static void gm_t_grp_expire(struct event *t)
 {
 	/* if we're here, that means when we received the group-specific query
 	 * there was one or more active S,G for this group.  For *,G the timer
@@ -1246,7 +1268,7 @@ static void gm_t_grp_expire(struct thread *t)
 	 * receive a report, so that work is left to gm_t_sg_expire and we
 	 * shouldn't worry about it here.
 	 */
-	struct gm_grp_pending *pend = THREAD_ARG(t);
+	struct gm_grp_pending *pend = EVENT_ARG(t);
 	struct gm_if *gm_ifp = pend->iface;
 	struct gm_sg *sg, *sg_start, sg_ref = {};
 
@@ -1275,7 +1297,7 @@ static void gm_t_grp_expire(struct thread *t)
 		 * parallel.  But if we received nothing for the *,G query,
 		 * the S,G query is kinda irrelevant.
 		 */
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 
 		frr_each_safe (gm_packet_sg_subs, sg->subs_positive, item)
 			/* this will also drop the EXCLUDE S,G lists */
@@ -1308,6 +1330,7 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 		if (PIM_DEBUG_GM_TRACE)
 			zlog_debug(log_ifp("*,%pPAs expiry timer starting"),
 				   &grp);
+		GM_UPDATE_SG_STATE(sg);
 		gm_sg_timer_start(gm_ifp, sg, timers->expire_wait);
 
 		sg = gm_sgs_next(gm_ifp->sgs, sg);
@@ -1322,11 +1345,11 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 	if (pend) {
 		struct timeval remain;
 
-		remain = thread_timer_remain(pend->t_expire);
+		remain = event_timer_remain(pend->t_expire);
 		if (timercmp(&remain, &timers->expire_wait, <=))
 			return;
 
-		THREAD_OFF(pend->t_expire);
+		EVENT_OFF(pend->t_expire);
 	} else {
 		pend = XCALLOC(MTYPE_GM_GRP_PENDING, sizeof(*pend));
 		pend->grp = grp;
@@ -1335,8 +1358,8 @@ static void gm_handle_q_group(struct gm_if *gm_ifp,
 	}
 
 	monotime(&pend->query);
-	thread_add_timer_tv(router->master, gm_t_grp_expire, pend,
-			    &timers->expire_wait, &pend->t_expire);
+	event_add_timer_tv(router->master, gm_t_grp_expire, pend,
+			   &timers->expire_wait, &pend->t_expire);
 
 	if (PIM_DEBUG_GM_TRACE)
 		zlog_debug(log_ifp("*,%pPAs S,G timer started: %pTHD"), &grp,
@@ -1347,7 +1370,7 @@ static void gm_bump_querier(struct gm_if *gm_ifp)
 {
 	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 
-	THREAD_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_query);
 
 	if (pim_addr_is_any(pim_ifp->ll_lowest))
 		return;
@@ -1356,12 +1379,12 @@ static void gm_bump_querier(struct gm_if *gm_ifp)
 
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
 
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
-static void gm_t_other_querier(struct thread *t)
+static void gm_t_other_querier(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 
 	zlog_info(log_ifp("other querier timer expired"));
@@ -1369,7 +1392,7 @@ static void gm_t_other_querier(struct thread *t)
 	gm_ifp->querier = pim_ifp->ll_lowest;
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
 
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
 static void gm_handle_query(struct gm_if *gm_ifp,
@@ -1472,13 +1495,12 @@ static void gm_handle_query(struct gm_if *gm_ifp,
 	if (IPV6_ADDR_CMP(&pkt_src->sin6_addr, &pim_ifp->ll_lowest) < 0) {
 		unsigned int other_ms;
 
-		THREAD_OFF(gm_ifp->t_query);
-		THREAD_OFF(gm_ifp->t_other_querier);
+		EVENT_OFF(gm_ifp->t_query);
+		EVENT_OFF(gm_ifp->t_other_querier);
 
 		other_ms = timers.qrv * timers.qqic_ms + timers.max_resp_ms / 2;
-		thread_add_timer_msec(router->master, gm_t_other_querier,
-				      gm_ifp, other_ms,
-				      &gm_ifp->t_other_querier);
+		event_add_timer_msec(router->master, gm_t_other_querier, gm_ifp,
+				     other_ms, &gm_ifp->t_other_querier);
 	}
 
 	if (len == sizeof(struct mld_v1_pkt)) {
@@ -1505,6 +1527,15 @@ static void gm_handle_query(struct gm_if *gm_ifp,
 		gm_handle_q_group(gm_ifp, &timers, hdr->grp);
 		gm_ifp->stats.rx_query_new_group++;
 	} else {
+		/* this is checked above:
+		 * if (len >= sizeof(struct mld_v2_query_hdr)) {
+		 *   size_t src_space = ntohs(hdr->n_src) * sizeof(pim_addr);
+		 *   if (len < sizeof(struct mld_v2_query_hdr) + src_space) {
+		 */
+		assume(ntohs(hdr->n_src) <=
+		       (len - sizeof(struct mld_v2_query_hdr)) /
+			       sizeof(pim_addr));
+
 		gm_handle_q_groupsrc(gm_ifp, &timers, hdr->grp, hdr->srcs,
 				     ntohs(hdr->n_src));
 		gm_ifp->stats.rx_query_new_groupsrc++;
@@ -1592,9 +1623,9 @@ static bool ip6_check_hopopts_ra(uint8_t *hopopts, size_t hopopt_len,
 	return false;
 }
 
-static void gm_t_recv(struct thread *t)
+static void gm_t_recv(struct event *t)
 {
-	struct pim_instance *pim = THREAD_ARG(t);
+	struct pim_instance *pim = EVENT_ARG(t);
 	union {
 		char buf[CMSG_SPACE(sizeof(struct in6_pktinfo)) +
 			 CMSG_SPACE(256) /* hop options */ +
@@ -1613,8 +1644,8 @@ static void gm_t_recv(struct thread *t)
 	ssize_t nread;
 	size_t pktlen;
 
-	thread_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
-			&pim->t_gm_recv);
+	event_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
+		       &pim->t_gm_recv);
 
 	iov->iov_base = rxbuf;
 	iov->iov_len = sizeof(rxbuf);
@@ -1645,18 +1676,6 @@ static void gm_t_recv(struct thread *t)
 		goto out_free;
 	}
 
-	struct interface *ifp;
-
-	ifp = if_lookup_by_index(pkt_src->sin6_scope_id, pim->vrf->vrf_id);
-	if (!ifp || !ifp->info)
-		goto out_free;
-
-	struct pim_interface *pim_ifp = ifp->info;
-	struct gm_if *gm_ifp = pim_ifp->mld;
-
-	if (!gm_ifp)
-		goto out_free;
-
 	for (cmsg = CMSG_FIRSTHDR(mh); cmsg; cmsg = CMSG_NXTHDR(mh, cmsg)) {
 		if (cmsg->cmsg_level != SOL_IPV6)
 			continue;
@@ -1675,6 +1694,21 @@ static void gm_t_recv(struct thread *t)
 		}
 	}
 
+	/* Prefer pktinfo as that also works in case of VRF */
+	ifindex_t ifindex = pktinfo ? pktinfo->ipi6_ifindex
+	                            : pkt_src->sin6_scope_id;
+	struct interface *ifp;
+
+	ifp = if_lookup_by_index(ifindex, pim->vrf->vrf_id);
+	if (!ifp || !ifp->info)
+		goto out_free;
+
+	struct pim_interface *pim_ifp = ifp->info;
+	struct gm_if *gm_ifp = pim_ifp->mld;
+
+	if (!gm_ifp)
+		goto out_free;
+
 	if (!pktinfo || !hoplimit) {
 		zlog_err(log_ifp(
 			"BUG: packet without IPV6_PKTINFO or IPV6_HOPLIMIT"));
@@ -1864,9 +1898,9 @@ static void gm_send_query(struct gm_if *gm_ifp, pim_addr grp,
 	}
 }
 
-static void gm_t_query(struct thread *t)
+static void gm_t_query(struct event *t)
 {
-	struct gm_if *gm_ifp = THREAD_ARG(t);
+	struct gm_if *gm_ifp = EVENT_ARG(t);
 	unsigned int timer_ms = gm_ifp->cur_query_intv;
 
 	if (gm_ifp->n_startup) {
@@ -1874,15 +1908,15 @@ static void gm_t_query(struct thread *t)
 		gm_ifp->n_startup--;
 	}
 
-	thread_add_timer_msec(router->master, gm_t_query, gm_ifp, timer_ms,
-			      &gm_ifp->t_query);
+	event_add_timer_msec(router->master, gm_t_query, gm_ifp, timer_ms,
+			     &gm_ifp->t_query);
 
 	gm_send_query(gm_ifp, PIMADDR_ANY, NULL, 0, false);
 }
 
-static void gm_t_sg_query(struct thread *t)
+static void gm_t_sg_query(struct event *t)
 {
-	struct gm_sg *sg = THREAD_ARG(t);
+	struct gm_sg *sg = EVENT_ARG(t);
 
 	gm_trigger_specific(sg);
 }
@@ -1901,9 +1935,9 @@ static void gm_send_specific(struct gm_gsq_pending *pend_gsq)
 	XFREE(MTYPE_GM_GSQ_PENDING, pend_gsq);
 }
 
-static void gm_t_gsq_pend(struct thread *t)
+static void gm_t_gsq_pend(struct event *t)
 {
-	struct gm_gsq_pending *pend_gsq = THREAD_ARG(t);
+	struct gm_gsq_pending *pend_gsq = EVENT_ARG(t);
 
 	gm_send_specific(pend_gsq);
 }
@@ -1911,17 +1945,28 @@ static void gm_t_gsq_pend(struct thread *t)
 static void gm_trigger_specific(struct gm_sg *sg)
 {
 	struct gm_if *gm_ifp = sg->iface;
-	struct pim_interface *pim_ifp = gm_ifp->ifp->info;
 	struct gm_gsq_pending *pend_gsq, ref = {};
 
 	sg->n_query--;
 	if (sg->n_query)
-		thread_add_timer_msec(router->master, gm_t_sg_query, sg,
-				      gm_ifp->cur_query_intv_trig,
-				      &sg->t_sg_query);
+		event_add_timer_msec(router->master, gm_t_sg_query, sg,
+				     gm_ifp->cur_query_intv_trig,
+				     &sg->t_sg_query);
+
+	/* As per RFC 2271, s6 p14:
+	 * E.g. a router that starts as a Querier, receives a
+	 * Done message for a group and then receives a Query from a router with
+	 * a lower address (causing a transition to the Non-Querier state)
+	 * continues to send multicast-address-specific queries for the group in
+	 * question until it either receives a Report or its timer expires, at
+	 * which time it starts performing the actions of a Non-Querier for this
+	 * group.
+	 */
+	 /* Therefore here we do not need to check if this router is querier or
+	  * not. This is called only for querier, hence it will work even if the
+	  * router transitions from querier to non-querier.
+	  */
 
-	if (!IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest))
-		return;
 	if (gm_ifp->pim->gm_socket == -1)
 		return;
 
@@ -1944,9 +1989,8 @@ static void gm_trigger_specific(struct gm_sg *sg)
 		pend_gsq->iface = gm_ifp;
 		gm_gsq_pends_add(gm_ifp->gsq_pends, pend_gsq);
 
-		thread_add_timer_tv(router->master, gm_t_gsq_pend, pend_gsq,
-				    &gm_ifp->cfg_timing_fuzz,
-				    &pend_gsq->t_send);
+		event_add_timer_tv(router->master, gm_t_gsq_pend, pend_gsq,
+				   &gm_ifp->cfg_timing_fuzz, &pend_gsq->t_send);
 	}
 
 	assert(pend_gsq->n_src < array_size(pend_gsq->srcs));
@@ -1955,7 +1999,7 @@ static void gm_trigger_specific(struct gm_sg *sg)
 	pend_gsq->n_src++;
 
 	if (pend_gsq->n_src == array_size(pend_gsq->srcs)) {
-		THREAD_OFF(pend_gsq->t_send);
+		EVENT_OFF(pend_gsq->t_send);
 		gm_send_specific(pend_gsq);
 		pend_gsq = NULL;
 	}
@@ -2052,8 +2096,8 @@ static void gm_vrf_socket_incref(struct pim_instance *pim)
 				vrf->name);
 	}
 
-	thread_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
-			&pim->t_gm_recv);
+	event_add_read(router->master, gm_t_recv, pim, pim->gm_socket,
+		       &pim->t_gm_recv);
 }
 
 static void gm_vrf_socket_decref(struct pim_instance *pim)
@@ -2061,7 +2105,7 @@ static void gm_vrf_socket_decref(struct pim_instance *pim)
 	if (--pim->gm_socket_if_count)
 		return;
 
-	THREAD_OFF(pim->t_gm_recv);
+	EVENT_OFF(pim->t_gm_recv);
 	close(pim->gm_socket);
 	pim->gm_socket = -1;
 }
@@ -2134,17 +2178,17 @@ void gm_group_delete(struct gm_if *gm_ifp)
 		gm_packet_drop(pkt, false);
 
 	while ((pend_grp = gm_grp_pends_pop(gm_ifp->grp_pends))) {
-		THREAD_OFF(pend_grp->t_expire);
+		EVENT_OFF(pend_grp->t_expire);
 		XFREE(MTYPE_GM_GRP_PENDING, pend_grp);
 	}
 
 	while ((pend_gsq = gm_gsq_pends_pop(gm_ifp->gsq_pends))) {
-		THREAD_OFF(pend_gsq->t_send);
+		EVENT_OFF(pend_gsq->t_send);
 		XFREE(MTYPE_GM_GSQ_PENDING, pend_gsq);
 	}
 
 	while ((sg = gm_sgs_pop(gm_ifp->sgs))) {
-		THREAD_OFF(sg->t_sg_expire);
+		EVENT_OFF(sg->t_sg_expire);
 		assertf(!gm_packet_sg_subs_count(sg->subs_negative), "%pSG",
 			&sg->sgaddr);
 		assertf(!gm_packet_sg_subs_count(sg->subs_positive), "%pSG",
@@ -2172,9 +2216,9 @@ void gm_ifp_teardown(struct interface *ifp)
 	if (PIM_DEBUG_GM_EVENTS)
 		zlog_debug(log_ifp("MLD stop"));
 
-	THREAD_OFF(gm_ifp->t_query);
-	THREAD_OFF(gm_ifp->t_other_querier);
-	THREAD_OFF(gm_ifp->t_expire);
+	EVENT_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_other_querier);
+	EVENT_OFF(gm_ifp->t_expire);
 
 	frr_with_privs (&pimd_privs) {
 		struct ipv6_mreq mreq;
@@ -2217,7 +2261,7 @@ static void gm_update_ll(struct interface *ifp)
 	gm_ifp->cur_ll_lowest = pim_ifp->ll_lowest;
 	if (was_querier)
 		gm_ifp->querier = pim_ifp->ll_lowest;
-	THREAD_OFF(gm_ifp->t_query);
+	EVENT_OFF(gm_ifp->t_query);
 
 	if (pim_addr_is_any(gm_ifp->cur_ll_lowest)) {
 		if (was_querier)
@@ -2238,7 +2282,7 @@ static void gm_update_ll(struct interface *ifp)
 		return;
 
 	gm_ifp->n_startup = gm_ifp->cur_qrv;
-	thread_execute(router->master, gm_t_query, gm_ifp, 0);
+	event_execute(router->master, gm_t_query, gm_ifp, 0, NULL);
 }
 
 void gm_ifp_update(struct interface *ifp)
@@ -2264,6 +2308,7 @@ void gm_ifp_update(struct interface *ifp)
 	if (!pim_ifp->mld) {
 		changed = true;
 		gm_start(ifp);
+		assume(pim_ifp->mld != NULL);
 	}
 
 	gm_ifp = pim_ifp->mld;
@@ -2319,9 +2364,7 @@ void gm_ifp_update(struct interface *ifp)
 
 #include "lib/command.h"
 
-#ifndef VTYSH_EXTRACT_PL
 #include "pimd/pim6_mld_clippy.c"
-#endif
 
 static struct vrf *gm_cmd_vrf_lookup(struct vty *vty, const char *vrf_str,
 				     int *err)
@@ -2395,24 +2438,20 @@ static void gm_show_if_one_detail(struct vty *vty, struct interface *ifp)
 }
 
 static void gm_show_if_one(struct vty *vty, struct interface *ifp,
-			   json_object *js_if)
+			   json_object *js_if, struct ttable *tt)
 {
 	struct pim_interface *pim_ifp = (struct pim_interface *)ifp->info;
 	struct gm_if *gm_ifp = pim_ifp->mld;
 	bool querier;
 
-	if (!gm_ifp) {
-		if (js_if)
-			json_object_string_add(js_if, "state", "down");
-		else
-			vty_out(vty, "%-16s  %5s\n", ifp->name, "down");
-		return;
-	}
+	assume(js_if || tt);
 
 	querier = IPV6_ADDR_SAME(&gm_ifp->querier, &pim_ifp->ll_lowest);
 
 	if (js_if) {
 		json_object_string_add(js_if, "name", ifp->name);
+		json_object_string_addf(js_if, "address", "%pPA",
+					&pim_ifp->primary_address);
 		json_object_string_add(js_if, "state", "up");
 		json_object_string_addf(js_if, "version", "%d",
 					gm_ifp->cur_version);
@@ -2439,11 +2478,11 @@ static void gm_show_if_one(struct vty *vty, struct interface *ifp,
 		json_object_int_add(js_if, "timerLastMemberQueryIntervalMsec",
 				    gm_ifp->cur_query_intv_trig);
 	} else {
-		vty_out(vty, "%-16s  %-5s  %d  %-25pPA  %-5s %11pTH  %pTVMs\n",
-			ifp->name, "up", gm_ifp->cur_version, &gm_ifp->querier,
-			querier ? "query" : "other",
-			querier ? gm_ifp->t_query : gm_ifp->t_other_querier,
-			&gm_ifp->started);
+		ttable_add_row(tt, "%s|%s|%pPAs|%d|%s|%pPAs|%pTH|%pTVMs",
+			       ifp->name, "up", &pim_ifp->primary_address,
+			       gm_ifp->cur_version, querier ? "local" : "other",
+			       &gm_ifp->querier, gm_ifp->t_query,
+			       &gm_ifp->started);
 	}
 }
 
@@ -2451,13 +2490,27 @@ static void gm_show_if_vrf(struct vty *vty, struct vrf *vrf, const char *ifname,
 			   bool detail, json_object *js)
 {
 	struct interface *ifp;
-	json_object *js_vrf;
+	json_object *js_vrf = NULL;
+	struct pim_interface *pim_ifp;
+	struct ttable *tt = NULL;
+	char *table = NULL;
 
 	if (js) {
 		js_vrf = json_object_new_object();
 		json_object_object_add(js, vrf->name, js_vrf);
 	}
 
+	if (!js && !detail) {
+		/* Prepare table. */
+		tt = ttable_new(&ttable_styles[TTSTYLE_BLANK]);
+		ttable_add_row(
+			tt,
+			"Interface|State|Address|V|Querier|QuerierIp|Query Timer|Uptime");
+		tt->style.cell.rpad = 2;
+		tt->style.corner = '+';
+		ttable_restyle(tt);
+	}
+
 	FOR_ALL_INTERFACES (vrf, ifp) {
 		json_object *js_if = NULL;
 
@@ -2468,24 +2521,44 @@ static void gm_show_if_vrf(struct vty *vty, struct vrf *vrf, const char *ifname,
 			continue;
 		}
 
-		if (!ifp->info)
+		pim_ifp = ifp->info;
+
+		if (!pim_ifp || !pim_ifp->mld)
 			continue;
+
 		if (js) {
 			js_if = json_object_new_object();
+			/*
+			 * If we have js as true and detail as false
+			 * and if Coverity thinks that js_if is NULL
+			 * because of a failed call to new then
+			 * when we call gm_show_if_one below
+			 * the tt can be deref'ed and as such
+			 * FRR will crash.  But since we know
+			 * that json_object_new_object never fails
+			 * then let's tell Coverity that this assumption
+			 * is true.  I'm not worried about fast path
+			 * here at all.
+			 */
+			assert(js_if);
 			json_object_object_add(js_vrf, ifp->name, js_if);
 		}
 
-		gm_show_if_one(vty, ifp, js_if);
+		gm_show_if_one(vty, ifp, js_if, tt);
+	}
+
+	/* Dump the generated table. */
+	if (!js && !detail) {
+		table = ttable_dump(tt, "\n");
+		vty_out(vty, "%s\n", table);
+		XFREE(MTYPE_TMP_TTABLE, table);
+		ttable_del(tt);
 	}
 }
 
 static void gm_show_if(struct vty *vty, struct vrf *vrf, const char *ifname,
 		       bool detail, json_object *js)
 {
-	if (!js && !detail)
-		vty_out(vty, "%-16s  %-5s  V  %-25s  %-18s  %s\n", "Interface",
-			"State", "Querier", "Timer", "Uptime");
-
 	if (vrf)
 		gm_show_if_vrf(vty, vrf, ifname, detail, js);
 	else
@@ -2739,7 +2812,7 @@ static void gm_show_joins_one(struct vty *vty, struct gm_if *gm_ifp,
 		}
 
 		js_src = json_object_new_object();
-		json_object_object_addf(js_group, js_src, "%pPA",
+		json_object_object_addf(js_group, js_src, "%pPAs",
 					&sg->sgaddr.src);
 
 		json_object_string_add(js_src, "state", gm_states[sg->state]);
@@ -2802,6 +2875,7 @@ static void gm_show_joins_vrf(struct vty *vty, struct vrf *vrf,
 
 	if (js) {
 		js_vrf = json_object_new_object();
+		json_object_string_add(js_vrf, "vrf", vrf->name);
 		json_object_object_add(js, vrf->name, js_vrf);
 	}
 
@@ -2961,7 +3035,7 @@ static void gm_show_groups(struct vty *vty, struct vrf *vrf, bool uj)
 		/* Dump the generated table. */
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 		ttable_del(tt);
 	}
 }
@@ -2997,9 +3071,9 @@ DEFPY(gm_debug_show,
       "debug show mld interface IFNAME",
       DEBUG_STR
       SHOW_STR
-      "MLD"
+      MLD_STR
       INTERFACE_STR
-      "interface name")
+      "interface name\n")
 {
 	struct interface *ifp;
 	struct pim_interface *pim_ifp;
