@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef PIM_IFCHANNEL_H
@@ -99,13 +86,13 @@ struct pim_ifchannel {
 
 	/* Per-interface (S,G) Join/Prune State (Section 4.1.4 of RFC4601) */
 	enum pim_ifjoin_state ifjoin_state;
-	struct thread *t_ifjoin_expiry_timer;
-	struct thread *t_ifjoin_prune_pending_timer;
+	struct event *t_ifjoin_expiry_timer;
+	struct event *t_ifjoin_prune_pending_timer;
 	int64_t ifjoin_creation; /* Record uptime of ifjoin state */
 
 	/* Per-interface (S,G) Assert State (Section 4.6.1 of RFC4601) */
 	enum pim_ifassert_state ifassert_state;
-	struct thread *t_ifassert_timer;
+	struct event *t_ifassert_timer;
 	pim_addr ifassert_winner;
 	struct pim_assert_metric ifassert_winner_metric;
 	int64_t ifassert_creation; /* Record uptime of ifassert state */
