@@ -13,6 +13,7 @@ DECLARE_MGROUP(PIMD);
 DECLARE_MTYPE(PIM_CHANNEL_OIL);
 DECLARE_MTYPE(PIM_INTERFACE);
 DECLARE_MTYPE(PIM_IGMP_JOIN);
+DECLARE_MTYPE(PIM_STATIC_GROUP);
 DECLARE_MTYPE(PIM_IGMP_SOCKET);
 DECLARE_MTYPE(PIM_IGMP_GROUP);
 DECLARE_MTYPE(PIM_IGMP_GROUP_SOURCE);
@@ -24,10 +25,12 @@ DECLARE_MTYPE(PIM_STATIC_ROUTE);
 DECLARE_MTYPE(PIM_RP);
 DECLARE_MTYPE(PIM_FILTER_NAME);
 DECLARE_MTYPE(PIM_MSDP_PEER);
+DECLARE_MTYPE(PIM_MSDP_FILTER_NAME);
 DECLARE_MTYPE(PIM_MSDP_MG_NAME);
 DECLARE_MTYPE(PIM_MSDP_SA);
 DECLARE_MTYPE(PIM_MSDP_MG);
 DECLARE_MTYPE(PIM_MSDP_MG_MBR);
+DECLARE_MTYPE(PIM_MSDP_AUTH_KEY);
 DECLARE_MTYPE(PIM_SEC_ADDR);
 DECLARE_MTYPE(PIM_JP_AGG_GROUP);
 DECLARE_MTYPE(PIM_JP_AGG_SOURCE);
