@@ -52,7 +52,7 @@ if args.dev_build:
                 "grep",
                 "-l",
                 "-P",
-                "^#\s*include.*_clippy.c",
+                r"^#\s*include.*_clippy.c",
                 "--",
                 "**.c",
             ]
@@ -91,7 +91,7 @@ lines = before.splitlines()
 autoderp = "#AUTODERP# "
 out_lines = []
 bcdeps = []
-make_rule_re = re.compile("^([^:\s]+):\s*([^:\s]+)\s*($|\n)")
+make_rule_re = re.compile(r"^([^:\s]+):\s*([^:\s]+)\s*($|\n)")
 
 while lines:
     line = lines.pop(0)
@@ -160,6 +160,17 @@ for clippy_file in clippy_scan:
 
 # combine daemon .xref files into frr.xref
 out_lines.append("")
+xref_targets = [
+    target
+    for target in xref_targets
+    if target
+    not in [
+        "bgpd/rfp-example/rfptest/rfptest",
+        "pimd/mtracebis",
+        "tools/ssd",
+        "vtysh/vtysh",
+    ]
+]
 out_lines.append(
     "xrefs = %s" % (" ".join(["%s.xref" % target for target in xref_targets]))
 )
