@@ -1263,9 +1263,13 @@ static void rip_response_process(struct rip_packet *packet, int size,
 					       rip->vrf->vrf_id)) {
 				struct route_node *rn;
 				struct rip_info *rinfo;
+				struct prefix p = { 0 };
 
-				rn = route_node_match_ipv4(rip->table,
-							   &rte->nexthop);
+				p.family = AF_INET;
+				p.prefixlen = IPV4_MAX_BITLEN;
+				p.u.prefix4 = rte->nexthop;
+
+				rn = route_node_match(rip->table, &p);
 
 				if (rn) {
 					rinfo = rn->info;
