@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * BGP Routing table range lookup test
  * Copyright (C) 2012 OSR.
@@ -5,20 +6,6 @@
  * Hamburg
  *
  * This file is part of FRRouting
- *
- * Quagga is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * Quagga is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -109,10 +96,7 @@ static void check_lookup_result(struct bgp_dest *match, va_list arglist)
 
 		if (bgp_dest_has_bgp_path_info_data(dest)
 		    && !prefix_in_array(dest_p, prefixes, prefix_count)) {
-			char buf[PREFIX2STR_BUFFER];
-
-			prefix2str(dest_p, buf, PREFIX2STR_BUFFER);
-			printf("prefix %s was not expected!\n", buf);
+			printf("prefix %pFX was not expected!\n", dest_p);
 			assert(0);
 		}
 	}
