@@ -13,6 +13,8 @@
  with route advertisements on a separate netns.
 """
 
+import json
+from functools import partial
 import os
 import sys
 import pytest
@@ -40,10 +42,12 @@ def build_topo(tgen):
 
     tgen.add_router("r1")
     tgen.add_router("r2")
+    tgen.add_router("r3")
 
     switch = tgen.add_switch("s1")
     switch.add_link(tgen.gears["r1"])
     switch.add_link(tgen.gears["r2"])
+    switch.add_link(tgen.gears["r3"])
 
     switch = tgen.add_switch("s2")
     switch.add_link(tgen.gears["r1"])
@@ -51,6 +55,9 @@ def build_topo(tgen):
     switch = tgen.add_switch("s3")
     switch.add_link(tgen.gears["r2"])
 
+    switch = tgen.add_switch("s4")
+    switch.add_link(tgen.gears["r3"])
+
 
 def setup_module(mod):
     "Sets up the pytest environment"
@@ -69,16 +76,16 @@ def setup_module(mod):
         )
         return pytest.skip("Skipping BGP EVPN RT5 NETNS Test. Kernel not supported")
 
-    # create VRF vrf-101 on R1 and R2
+    # create VRF vrf-101 on R1, R2, R3
     # create loop101
     cmds_vrflite = [
-        "ip link add {}-vrf-101 type vrf table 101",
-        "ip ru add oif {}-vrf-101 table 101",
-        "ip ru add iif {}-vrf-101 table 101",
-        "ip link set dev {}-vrf-101 up",
-        "ip link add loop101 type dummy",
-        "ip link set dev loop101 master {}-vrf-101",
-        "ip link set dev loop101 up",
+        "ip link add {0}-vrf-{1} type vrf table {1}",
+        "ip ru add oif {0}-vrf-{1} table {1}",
+        "ip ru add iif {0}-vrf-{1} table {1}",
+        "ip link set dev {0}-vrf-{1} up",
+        "ip link add loop{1} type dummy",
+        "ip link set dev loop{1} master {0}-vrf-{1}",
+        "ip link set dev loop{1} up",
     ]
 
     cmds_r2 = [  # config routing 101
@@ -90,6 +97,15 @@ def setup_module(mod):
         "ip link set vxlan-101 up type bridge_slave learning off flood off mcast_flood off",
     ]
 
+    cmds_r3 = [  # config routing 102
+        "ip link add name bridge-102 up type bridge stp_state 0",
+        "ip link set bridge-102 master {}-vrf-102",
+        "ip link set dev bridge-102 up",
+        "ip link add name vxlan-102 type vxlan id 102 dstport 4789 dev r3-eth0 local 192.168.100.61",
+        "ip link set dev vxlan-102 master bridge-102",
+        "ip link set vxlan-102 up type bridge_slave learning off flood off mcast_flood off",
+    ]
+
     # cmds_r1_netns_method3 = [
     #     "ip link add name vxlan-{1} type vxlan id {1} dstport 4789 dev {0}-eth0 local 192.168.100.21",
     #     "ip link set dev vxlan-{1} netns {0}-vrf-{1}",
@@ -109,8 +125,8 @@ def setup_module(mod):
 
     router = tgen.gears["r2"]
     for cmd in cmds_vrflite:
-        logger.info("cmd to r2: " + cmd.format("r2"))
-        output = router.cmd_raises(cmd.format("r2"))
+        logger.info("cmd to r2: " + cmd.format("r2", 101))
+        output = router.cmd_raises(cmd.format("r2", 101))
         logger.info("result: " + output)
 
     for cmd in cmds_r2:
@@ -118,6 +134,17 @@ def setup_module(mod):
         output = router.cmd_raises(cmd.format("r2"))
         logger.info("result: " + output)
 
+    router = tgen.gears["r3"]
+    for cmd in cmds_vrflite:
+        logger.info("cmd to r3: " + cmd.format("r3", 102))
+        output = router.cmd_raises(cmd.format("r3", 102))
+        logger.info("result: " + output)
+
+    for cmd in cmds_r3:
+        logger.info("cmd to r3: " + cmd.format("r3"))
+        output = router.cmd_raises(cmd.format("r3"))
+        logger.info("result: " + output)
+
     tgen.net["r1"].cmd_raises(
         "ip link add name vxlan-101 type vxlan id 101 dstport 4789 dev r1-eth0 local 192.168.100.21"
     )
@@ -132,21 +159,13 @@ def setup_module(mod):
     tgen.net["r1"].cmd_raises("ip -n r1-vrf-101 link set bridge-101 up")
     tgen.net["r1"].cmd_raises("ip -n r1-vrf-101 link set vxlan-101 up")
 
-    for rname, router in router_list.items():
+    for rname, router in tgen.routers().items():
+        logger.info("Loading router %s" % rname)
         if rname == "r1":
-            router.load_config(TopoRouter.RD_MGMTD, None, "--vrfwnetns")
-            router.load_config(
-                TopoRouter.RD_ZEBRA,
-                os.path.join(CWD, "{}/zebra.conf".format(rname)),
-                "--vrfwnetns",
-            )
+            router.use_netns_vrf()
+            router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))
         else:
-            router.load_config(
-                TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
-            )
-        router.load_config(
-            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
-        )
+            router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))
 
     # Initialize all routers.
     tgen.start_router()
@@ -160,6 +179,36 @@ def teardown_module(_mod):
     tgen.stop_topology()
 
 
+def _test_evpn_ping_router(pingrouter, ipv4_only=False):
+    """
+    internal function to check ping between r1 and r2
+    """
+    # Check IPv4 and IPv6 connectivity between r1 and r2 ( routing vxlan evpn)
+    logger.info(
+        "Check Ping IPv4 from  R1(r1-vrf-101) to R2(r2-vrf-101 = 192.168.101.41)"
+    )
+    output = pingrouter.run("ip netns exec r1-vrf-101 ping 192.168.101.41 -f -c 1000")
+    logger.info(output)
+    if "1000 packets transmitted, 1000 received" not in output:
+        assertmsg = (
+            "expected ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) should be ok"
+        )
+        assert 0, assertmsg
+    else:
+        logger.info("Check Ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) OK")
+
+    if ipv4_only:
+        return
+
+    logger.info("Check Ping IPv6 from  R1(r1-vrf-101) to R2(r2-vrf-101 = fd00::2)")
+    output = pingrouter.run("ip netns exec r1-vrf-101 ping fd00::2 -f -c 1000")
+    logger.info(output)
+    if "1000 packets transmitted, 1000 received" not in output:
+        assert 0, "expected ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) should be ok"
+    else:
+        logger.info("Check Ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) OK")
+
+
 def test_protocols_convergence():
     """
     Assert that all protocols have converged
@@ -168,7 +217,34 @@ def test_protocols_convergence():
     tgen = get_topogen()
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
-    topotest.sleep(4, "waiting 4 seconds for bgp convergence")
+    # Check BGP IPv4 routing tables on r1
+    logger.info("Checking BGP L2VPN EVPN routes for convergence on r1")
+
+    for rname in ("r1", "r2"):
+        router = tgen.gears[rname]
+        json_file = "{}/{}/bgp_l2vpn_evpn_routes.json".format(CWD, router.name)
+        if not os.path.isfile(json_file):
+            assert 0, "bgp_l2vpn_evpn_routes.json file not found"
+
+        expected = json.loads(open(json_file).read())
+        test_func = partial(
+            topotest.router_json_cmp,
+            router,
+            "show bgp l2vpn evpn json",
+            expected,
+        )
+        _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
+        assertmsg = '"{}" JSON output mismatches'.format(router.name)
+        assert result is None, assertmsg
+
+
+def test_protocols_dump_info():
+    """
+    Dump EVPN information
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
     # Check IPv4/IPv6 routing tables.
     output = tgen.gears["r1"].vtysh_cmd("show bgp l2vpn evpn", isjson=False)
     logger.info("==== result from show bgp l2vpn evpn")
@@ -203,6 +279,15 @@ def test_protocols_convergence():
     logger.info("==== result from show evpn rmac vni all")
     logger.info(output)
 
+
+def test_router_check_ip():
+    """
+    Check routes are correctly installed
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
+
     expected = {
         "fd00::2/128": [
             {
@@ -210,7 +295,7 @@ def test_protocols_convergence():
                 "vrfName": "r1-vrf-101",
                 "nexthops": [
                     {
-                        "ip": "::ffff:c0a8:6429",
+                        "ip": "::ffff:192.168.100.41",
                     }
                 ],
             }
@@ -221,56 +306,112 @@ def test_protocols_convergence():
     )
     assert result is None, "ipv6 route check failed"
 
-    expected = {
-        "101": {
-            "numNextHops": 2,
-            "192.168.100.41": {
-                "nexthopIp": "192.168.100.41",
-            },
-            "::ffff:c0a8:6429": {
-                "nexthopIp": "::ffff:c0a8:6429",
-            },
+
+def _test_router_check_evpn_contexts(router, ipv4_only=False):
+    """
+    Check EVPN nexthops and RMAC number  are correctly configured
+    """
+    if ipv4_only:
+        expected = {
+            "101": {
+                "numNextHops": 1,
+                "192.168.100.41": {
+                    "nexthopIp": "192.168.100.41",
+                },
+            }
+        }
+    else:
+        expected = {
+            "101": {
+                "numNextHops": 2,
+                "192.168.100.41": {
+                    "nexthopIp": "192.168.100.41",
+                },
+                "::ffff:192.168.100.41": {
+                    "nexthopIp": "::ffff:192.168.100.41",
+                },
+            }
         }
-    }
     result = topotest.router_json_cmp(
-        tgen.gears["r1"], "show evpn next-hops vni all json", expected
+        router, "show evpn next-hops vni all json", expected
     )
     assert result is None, "evpn next-hops check failed"
 
     expected = {"101": {"numRmacs": 1}}
-    result = topotest.router_json_cmp(
-        tgen.gears["r1"], "show evpn rmac vni all json", expected
-    )
+    result = topotest.router_json_cmp(router, "show evpn rmac vni all json", expected)
     assert result is None, "evpn rmac number check failed"
 
-    # Check IPv4 and IPv6 connectivity between r1 and r2 ( routing vxlan evpn)
-    pingrouter = tgen.gears["r1"]
-    logger.info(
-        "Check Ping IPv4 from  R1(r1-vrf-101) to R2(r2-vrf-101 = 192.168.101.41)"
-    )
-    output = pingrouter.run("ip netns exec r1-vrf-101 ping 192.168.101.41 -f -c 1000")
-    logger.info(output)
-    if "1000 packets transmitted, 1000 received" not in output:
-        assertmsg = (
-            "expected ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) should be ok"
-        )
-        assert 0, assertmsg
-    else:
-        logger.info("Check Ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) OK")
 
-    logger.info("Check Ping IPv6 from  R1(r1-vrf-101) to R2(r2-vrf-101 = fd00::2)")
-    output = pingrouter.run("ip netns exec r1-vrf-101 ping fd00::2 -f -c 1000")
-    logger.info(output)
-    if "1000 packets transmitted, 1000 received" not in output:
-        assert 0, "expected ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) should be ok"
-    else:
-        logger.info("Check Ping IPv6 from R1(r1-vrf-101) to R2(fd00::2) OK")
+def test_router_check_evpn_contexts():
+    """
+    Check EVPN nexthops and RMAC number  are correctly configured
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
+
+    _test_router_check_evpn_contexts(tgen.gears["r1"])
+
+
+def test_evpn_ping():
+    """
+    Check ping between R1 and R2 is ok
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
+
+    _test_evpn_ping_router(tgen.gears["r1"])
+
+
+def test_evpn_disable_routemap():
+    """
+    Check the removal of a route-map on R2. More EVPN Prefixes are expected
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
+
+    tgen.gears["r2"].vtysh_cmd(
+        """
+        configure terminal\n
+        router bgp 65000 vrf r2-vrf-101\n
+        address-family l2vpn evpn\n
+        advertise ipv4 unicast\n
+        advertise ipv6 unicast\n
+        """
+    )
+    router = tgen.gears["r1"]
+    json_file = "{}/{}/bgp_l2vpn_evpn_routes_all.json".format(CWD, router.name)
+    if not os.path.isfile(json_file):
+        assert 0, "bgp_l2vpn_evpn_routes.json file not found"
+
+    expected = json.loads(open(json_file).read())
+    test_func = partial(
+        topotest.router_json_cmp,
+        router,
+        "show bgp l2vpn evpn json",
+        expected,
+    )
+    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
+    assertmsg = '"{}" JSON output mismatches'.format(router.name)
+    assert result is None, assertmsg
+
+
+def test_evpn_remove_ip():
+    """
+    Check the removal of an EVPN route is correctly handled
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
 
     config_no_ipv6 = {
         "r2": {
             "raw_config": [
                 "router bgp 65000 vrf r2-vrf-101",
                 "address-family ipv6 unicast",
+                "no network fd00::3/128",
                 "no network fd00::2/128",
             ]
         }
@@ -293,6 +434,7 @@ def test_protocols_convergence():
     }
     result = verify_bgp_rib(tgen, "ipv6", "r1", ipv6_routes, expected=False)
     assert result is not True, "expect IPv6 route fd00::2/128 withdrawn"
+
     output = tgen.gears["r1"].vtysh_cmd("show evpn next-hops vni all", isjson=False)
     logger.info("==== result from show evpn next-hops vni all")
     logger.info(output)
@@ -300,37 +442,27 @@ def test_protocols_convergence():
     logger.info("==== result from show evpn next-hops vni all")
     logger.info(output)
 
-    expected = {
-        "101": {
-            "numNextHops": 1,
-            "192.168.100.41": {
-                "nexthopIp": "192.168.100.41",
-            },
-        }
-    }
-    result = topotest.router_json_cmp(
-        tgen.gears["r1"], "show evpn next-hops vni all json", expected
-    )
-    assert result is None, "evpn next-hops check failed"
 
-    expected = {"101": {"numRmacs": 1}}
-    result = topotest.router_json_cmp(
-        tgen.gears["r1"], "show evpn rmac vni all json", expected
-    )
-    assert result is None, "evpn rmac number check failed"
+def test_router_check_evpn_contexts_again():
+    """
+    Check EVPN nexthops and RMAC number  are correctly configured
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
 
-    logger.info(
-        "Check Ping IPv4 from  R1(r1-vrf-101) to R2(r2-vrf-101 = 192.168.101.41)"
-    )
-    output = pingrouter.run("ip netns exec r1-vrf-101 ping 192.168.101.41 -f -c 1000")
-    logger.info(output)
-    if "1000 packets transmitted, 1000 received" not in output:
-        assertmsg = (
-            "expected ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) should be ok"
-        )
-        assert 0, assertmsg
-    else:
-        logger.info("Check Ping IPv4 from R1(r1-vrf-101) to R2(192.168.101.41) OK")
+    _test_router_check_evpn_contexts(tgen.gears["r1"], ipv4_only=True)
+
+
+def test_evpn_ping_again():
+    """
+    Check ping between R1 and R2 is ok
+    """
+    tgen = get_topogen()
+    if tgen.routers_have_failure():
+        pytest.skip(tgen.errors)
+
+    _test_evpn_ping_router(tgen.gears["r1"], ipv4_only=True)
 
 
 def test_memory_leak():
