@@ -670,7 +670,7 @@ def test_configure_gua_on_unnumbered_intf(request):
                 {
                     "nexthops": [
                         {
-                            "ip": "::ffff:a00:501",
+                            "ip": "::ffff:10.0.5.1",
                             "hostname": "r1",
                             "afi": "ipv6",
                             "scope": "global",
@@ -754,7 +754,7 @@ def test_configure_gua_on_unnumbered_intf(request):
     assert (
         result is None
     ), "Testcase {} : Failed \n Error: Nexthop for prefix 11.0.20.1 \
-    is not ::ffff:a00:501".format(
+    is not ::ffff:10.0.5.1".format(
         tc_name
     )
 
