@@ -18,7 +18,7 @@ import json
 import pytest
 import functools
 
-pytestmark = pytest.mark.bgpd
+pytestmark = [pytest.mark.bgpd]
 
 CWD = os.path.dirname(os.path.realpath(__file__))
 sys.path.append(os.path.join(CWD, "../"))
@@ -27,8 +27,6 @@ sys.path.append(os.path.join(CWD, "../"))
 from lib import topotest
 from lib.topogen import Topogen, TopoRouter, get_topogen
 
-pytestmark = [pytest.mark.bgpd]
-
 
 def build_topo(tgen):
     for routern in range(1, 3):
@@ -45,7 +43,7 @@ def setup_module(mod):
 
     router_list = tgen.routers()
 
-    for i, (rname, router) in enumerate(router_list.items(), 1):
+    for _, (rname, router) in enumerate(router_list.items(), 1):
         router.load_config(
             TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
         )
@@ -88,7 +86,7 @@ def test_bgp_extcommunity_none():
         return topotest.json_cmp(output, expected)
 
     test_func = functools.partial(_bgp_converge, router)
-    success, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
+    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
     assert result is None, "BGP Converge failed"
 
     def _bgp_extcommunity_strip(router):
@@ -113,7 +111,7 @@ def test_bgp_extcommunity_none():
         return topotest.json_cmp(output, expected)
 
     test_func = functools.partial(_bgp_extcommunity_strip, router)
-    success, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
+    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
     assert result is None, "Failed to strip incoming extended communities from r2"
 
 
