@@ -6,10 +6,8 @@
 #
 
 import os
-import re
 import sys
 import json
-import functools
 import pytest
 
 CWD = os.path.dirname(os.path.realpath(__file__))
@@ -20,7 +18,6 @@ sys.path.append(os.path.join(CWD, "../"))
 from lib import topotest
 from lib.topogen import Topogen, TopoRouter, get_topogen
 from lib.topolog import logger
-from lib.common_config import required_linux_kernel_version
 
 pytestmark = [pytest.mark.bgpd]
 
@@ -37,10 +34,12 @@ def setup_module(mod):
     tgen.start_topology()
 
     for rname, router in tgen.routers().items():
-        router.load_config(TopoRouter.RD_ZEBRA,
-                           os.path.join(CWD, '{}/zebra.conf'.format(rname)))
-        router.load_config(TopoRouter.RD_BGP,
-                           os.path.join(CWD, '{}/bgpd.conf'.format(rname)))
+        router.load_config(
+            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
+        )
+        router.load_config(
+            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
+        )
 
     tgen.gears["pe1"].run("ip link add vrf10 type vrf table 10")
     tgen.gears["pe1"].run("ip link set vrf10 up")
@@ -62,22 +61,22 @@ def open_json_file(path):
             return json.load(f)
     except IOError:
         assert False, "Could not read file {}".format(path)
-    
+
 
 def check_vrf10_rib(output):
-    expected = open_json_file("%s/pe1/results/vrf10_ipv4_unicast.json" % CWD) 
+    expected = open_json_file("%s/pe1/results/vrf10_ipv4_unicast.json" % CWD)
     actual = json.loads(output)
     return topotest.json_cmp(actual, expected)
 
 
 def check_default_vpn_rib(output):
-    expected = open_json_file("%s/pe1/results/default_ipv4_vpn.json" % CWD) 
+    expected = open_json_file("%s/pe1/results/default_ipv4_vpn.json" % CWD)
     actual = json.loads(output)
     return topotest.json_cmp(actual, expected)
 
 
 def check_vrf20_rib(output):
-    expected = open_json_file("%s/pe1/results/vrf20_ipv4_unicast.json" % CWD) 
+    expected = open_json_file("%s/pe1/results/vrf20_ipv4_unicast.json" % CWD)
     actual = json.loads(output)
     return topotest.json_cmp(actual, expected)
 
