@@ -55,7 +55,6 @@ from lib.common_config import (
     apply_raw_config,
     create_static_routes,
     required_linux_kernel_version,
-    topo_daemons,
 )
 from lib.pim import (
     create_pim_config,
@@ -191,7 +190,7 @@ def reset_stats(stats):
     """
 
     for router, state_data in stats.items():
-        for state, value in state_data.items():
+        for state, _ in state_data.items():
             stats[router][state] = 0
             logger.info(
                 "[DUT: %s]: stats %s value has reset" " reset, Current value: %s",
@@ -214,7 +213,7 @@ def verify_state_incremented(state_before, state_after):
     """
 
     for router, state_data in state_before.items():
-        for state, value in state_data.items():
+        for state, _ in state_data.items():
             if state_before[router][state] >= state_after[router][state]:
                 errormsg = (
                     "[DUT: %s]: state %s value has not"
