@@ -1,25 +1,17 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Monitor status of frr daemons and restart if necessary.
  *
  * Copyright (C) 2004  Andrew J. Schorr
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
-#include <thread.h>
+
+#include <signal.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#include "frrevent.h"
 #include <log.h>
 #include <network.h>
 #include <sigevent.h>
@@ -40,6 +32,8 @@
 #include "watchfrr.h"
 #include "watchfrr_errors.h"
 
+#include "lib/config_paths.h"
+
 #ifndef MIN
 #define MIN(X,Y) (((X) <= (Y)) ? (X) : (Y))
 #endif
@@ -50,7 +44,7 @@
 
 #define DEFAULT_PERIOD		5
 #define DEFAULT_TIMEOUT		90
-#define DEFAULT_RESTART_TIMEOUT	20
+#define DEFAULT_RESTART_TIMEOUT	    90
 #define DEFAULT_LOGLEVEL	LOG_INFO
 #define DEFAULT_MIN_RESTART	60
 #define DEFAULT_MAX_RESTART	600
@@ -66,7 +60,7 @@ DEFINE_MGROUP(WATCHFRR, "watchfrr");
 DEFINE_MTYPE_STATIC(WATCHFRR, WATCHFRR_DAEMON, "watchfrr daemon entry");
 
 /* Needs to be global, referenced somewhere inside libfrr. */
-struct thread_master *master;
+struct event_loop *master;
 
 static bool watch_only = false;
 const char *pathspace;
@@ -99,19 +93,20 @@ struct restart_info {
 	pid_t pid;
 	struct timeval time;
 	long interval;
-	struct thread *t_kill;
+	struct event *t_kill;
 	int kills;
 };
 
 static struct global_state {
 	enum restart_phase phase;
-	struct thread *t_phase_hanging;
-	struct thread *t_startup_timeout;
-	struct thread *t_operational;
+	struct event *t_phase_hanging;
+	struct event *t_startup_timeout;
+	struct event *t_operational;
 	const char *vtydir;
 	long period;
 	long timeout;
 	long restart_timeout;
+	bool reading_configuration;
 	long min_restart_interval;
 	long max_restart_interval;
 	long operational_timeout;
@@ -127,7 +122,7 @@ static struct global_state {
 	int numdown; /* # of daemons that are not UP or UNRESPONSIVE */
 } gs = {
 	.phase = PHASE_INIT,
-	.vtydir = frr_vtydir,
+	.vtydir = frr_runstatedir,
 	.period = 1000 * DEFAULT_PERIOD,
 	.timeout = DEFAULT_TIMEOUT,
 	.restart_timeout = DEFAULT_RESTART_TIMEOUT,
@@ -161,9 +156,9 @@ struct daemon {
 	int fd;
 	struct timeval echo_sent;
 	unsigned int connect_tries;
-	struct thread *t_wakeup;
-	struct thread *t_read;
-	struct thread *t_write;
+	struct event *t_wakeup;
+	struct event *t_read;
+	struct event *t_write;
 	struct daemon *next;
 	struct restart_info restart;
 
@@ -207,7 +202,7 @@ static const struct option longopts[] = {
 	{NULL, 0, NULL, 0}};
 
 static int try_connect(struct daemon *dmn);
-static void wakeup_send_echo(struct thread *t_wakeup);
+static void wakeup_send_echo(struct event *t_wakeup);
 static void try_restart(struct daemon *dmn);
 static void phase_check(void);
 static void restart_done(struct daemon *dmn);
@@ -304,11 +299,11 @@ Otherwise, the interval is doubled (but capped at the -M value).\n\n",
 		passing command-line arguments with embedded spaces.\n\
 -v, --version	Print program version\n\
 -h, --help	Display this help and exit\n",
-		frr_vtydir, DEFAULT_LOGLEVEL, LOG_EMERG, LOG_DEBUG, LOG_DEBUG,
-		DEFAULT_MIN_RESTART, DEFAULT_MAX_RESTART,
+		frr_runstatedir, DEFAULT_LOGLEVEL, LOG_EMERG, LOG_DEBUG,
+		LOG_DEBUG, DEFAULT_MIN_RESTART, DEFAULT_MAX_RESTART,
 		DEFAULT_OPERATIONAL_TIMEOUT, DEFAULT_PERIOD, DEFAULT_TIMEOUT,
 		DEFAULT_RESTART_TIMEOUT, DEFAULT_RESTART_CMD, DEFAULT_START_CMD,
-		DEFAULT_STOP_CMD, frr_vtydir);
+		DEFAULT_STOP_CMD, frr_runstatedir);
 }
 
 static pid_t run_background(char *shell_cmd)
@@ -359,20 +354,30 @@ static struct timeval *time_elapsed(struct timeval *result,
 	return result;
 }
 
-static void restart_kill(struct thread *t_kill)
+static void restart_kill(struct event *t_kill)
 {
-	struct restart_info *restart = THREAD_ARG(t_kill);
+	struct restart_info *restart = EVENT_ARG(t_kill);
 	struct timeval delay;
 
 	time_elapsed(&delay, &restart->time);
+
+	if (gs.reading_configuration) {
+		zlog_err(
+			"%s %s child process appears to still be reading configuration, delaying for another %lu time",
+			restart->what, restart->name, gs.restart_timeout);
+		event_add_timer(master, restart_kill, restart,
+				gs.restart_timeout, &restart->t_kill);
+		return;
+	}
+
 	zlog_warn(
 		"%s %s child process %d still running after %ld seconds, sending signal %d",
 		restart->what, restart->name, (int)restart->pid,
 		(long)delay.tv_sec, (restart->kills ? SIGKILL : SIGTERM));
 	kill(-restart->pid, (restart->kills ? SIGKILL : SIGTERM));
 	restart->kills++;
-	thread_add_timer(master, restart_kill, restart, gs.restart_timeout,
-			 &restart->t_kill);
+	event_add_timer(master, restart_kill, restart, gs.restart_timeout,
+			&restart->t_kill);
 }
 
 static struct restart_info *find_child(pid_t child)
@@ -417,7 +422,7 @@ static void sigchild(void)
 		what = restart->what;
 		restart->pid = 0;
 		gs.numpids--;
-		thread_cancel(&restart->t_kill);
+		event_cancel(&restart->t_kill);
 
 		/* Update restart time to reflect the time the command
 		 * completed. */
@@ -501,10 +506,14 @@ static int run_job(struct restart_info *restart, const char *cmdtype,
 	restart->kills = 0;
 	{
 		char cmd[strlen(command) + strlen(restart->name) + 1];
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+		/* user supplied command string has a %s for the daemon name */
 		snprintf(cmd, sizeof(cmd), command, restart->name);
+#pragma GCC diagnostic pop
 		if ((restart->pid = run_background(cmd)) > 0) {
-			thread_add_timer(master, restart_kill, restart,
-					 gs.restart_timeout, &restart->t_kill);
+			event_add_timer(master, restart_kill, restart,
+					gs.restart_timeout, &restart->t_kill);
 			restart->what = cmdtype;
 			gs.numpids++;
 		} else
@@ -527,34 +536,34 @@ static int run_job(struct restart_info *restart, const char *cmdtype,
 #define SET_READ_HANDLER(DMN)                                                  \
 	do {                                                                   \
 		(DMN)->t_read = NULL;                                          \
-		thread_add_read(master, handle_read, (DMN), (DMN)->fd,         \
-				&(DMN)->t_read);                               \
+		event_add_read(master, handle_read, (DMN), (DMN)->fd,          \
+			       &(DMN)->t_read);                                \
 	} while (0);
 
 #define SET_WAKEUP_DOWN(DMN)                                                   \
 	do {                                                                   \
 		(DMN)->t_wakeup = NULL;                                        \
-		thread_add_timer_msec(master, wakeup_down, (DMN),              \
-				      FUZZY(gs.period), &(DMN)->t_wakeup);     \
+		event_add_timer_msec(master, wakeup_down, (DMN),               \
+				     FUZZY(gs.period), &(DMN)->t_wakeup);      \
 	} while (0);
 
 #define SET_WAKEUP_UNRESPONSIVE(DMN)                                           \
 	do {                                                                   \
 		(DMN)->t_wakeup = NULL;                                        \
-		thread_add_timer_msec(master, wakeup_unresponsive, (DMN),      \
-				      FUZZY(gs.period), &(DMN)->t_wakeup);     \
+		event_add_timer_msec(master, wakeup_unresponsive, (DMN),       \
+				     FUZZY(gs.period), &(DMN)->t_wakeup);      \
 	} while (0);
 
 #define SET_WAKEUP_ECHO(DMN)                                                   \
 	do {                                                                   \
 		(DMN)->t_wakeup = NULL;                                        \
-		thread_add_timer_msec(master, wakeup_send_echo, (DMN),         \
-				      FUZZY(gs.period), &(DMN)->t_wakeup);     \
+		event_add_timer_msec(master, wakeup_send_echo, (DMN),          \
+				     FUZZY(gs.period), &(DMN)->t_wakeup);      \
 	} while (0);
 
-static void wakeup_down(struct thread *t_wakeup)
+static void wakeup_down(struct event *t_wakeup)
 {
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 
 	dmn->t_wakeup = NULL;
 	if (try_connect(dmn) < 0)
@@ -563,9 +572,9 @@ static void wakeup_down(struct thread *t_wakeup)
 		try_restart(dmn);
 }
 
-static void wakeup_init(struct thread *t_wakeup)
+static void wakeup_init(struct event *t_wakeup)
 {
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 
 	dmn->t_wakeup = NULL;
 	if (try_connect(dmn) < 0) {
@@ -585,19 +594,22 @@ static void restart_done(struct daemon *dmn)
 			dmn->name, state_str[dmn->state]);
 		return;
 	}
-	THREAD_OFF(dmn->t_wakeup);
+	EVENT_OFF(dmn->t_wakeup);
 
 	if (try_connect(dmn) < 0)
 		SET_WAKEUP_DOWN(dmn);
 }
 
-static void daemon_restarting_operational(struct thread *thread)
+static void daemon_restarting_operational(struct event *thread)
 {
 	systemd_send_status("FRR Operational");
 }
 
 static void daemon_down(struct daemon *dmn, const char *why)
 {
+	if (dmn->ignore_timeout)
+		return;
+
 	if (IS_UP(dmn) || (dmn->state == DAEMON_INIT))
 		flog_err(EC_WATCHFRR_CONNECTION, "%s state -> down : %s",
 			 dmn->name, why);
@@ -610,9 +622,9 @@ static void daemon_down(struct daemon *dmn, const char *why)
 		close(dmn->fd);
 		dmn->fd = -1;
 	}
-	THREAD_OFF(dmn->t_read);
-	THREAD_OFF(dmn->t_write);
-	THREAD_OFF(dmn->t_wakeup);
+	EVENT_OFF(dmn->t_read);
+	EVENT_OFF(dmn->t_write);
+	EVENT_OFF(dmn->t_wakeup);
 	if (try_connect(dmn) < 0)
 		SET_WAKEUP_DOWN(dmn);
 
@@ -620,9 +632,9 @@ static void daemon_down(struct daemon *dmn, const char *why)
 	phase_check();
 }
 
-static void handle_read(struct thread *t_read)
+static void handle_read(struct event *t_read)
 {
-	struct daemon *dmn = THREAD_ARG(t_read);
+	struct daemon *dmn = EVENT_ARG(t_read);
 	static const char resp[sizeof(PING_TOKEN) + 4] = PING_TOKEN "\n";
 	char buf[sizeof(resp) + 100];
 	ssize_t rc;
@@ -686,7 +698,7 @@ static void handle_read(struct thread *t_read)
 			   dmn->name, (long)delay.tv_sec, (long)delay.tv_usec);
 
 	SET_READ_HANDLER(dmn);
-	thread_cancel(&dmn->t_wakeup);
+	event_cancel(&dmn->t_wakeup);
 	SET_WAKEUP_ECHO(dmn);
 }
 
@@ -718,7 +730,7 @@ static void daemon_send_ready(int exitcode)
 
 	frr_detach();
 
-	snprintf(started, sizeof(started), "%s/%s", frr_vtydir,
+	snprintf(started, sizeof(started), "%s/%s", frr_runstatedir,
 		 "watchfrr.started");
 	fp = fopen(started, "w");
 	if (fp)
@@ -738,19 +750,19 @@ static void daemon_up(struct daemon *dmn, const char *why)
 	if (gs.numdown == 0) {
 		daemon_send_ready(0);
 
-		THREAD_OFF(gs.t_operational);
+		EVENT_OFF(gs.t_operational);
 
-		thread_add_timer(master, daemon_restarting_operational, NULL,
-				 gs.operational_timeout, &gs.t_operational);
+		event_add_timer(master, daemon_restarting_operational, NULL,
+				gs.operational_timeout, &gs.t_operational);
 	}
 
 	SET_WAKEUP_ECHO(dmn);
 	phase_check();
 }
 
-static void check_connect(struct thread *t_write)
+static void check_connect(struct event *t_write)
 {
-	struct daemon *dmn = THREAD_ARG(t_write);
+	struct daemon *dmn = EVENT_ARG(t_write);
 	int sockerr;
 	socklen_t reslen = sizeof(sockerr);
 
@@ -776,9 +788,9 @@ static void check_connect(struct thread *t_write)
 	daemon_up(dmn, "delayed connect succeeded");
 }
 
-static void wakeup_connect_hanging(struct thread *t_wakeup)
+static void wakeup_connect_hanging(struct event *t_wakeup)
 {
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 	char why[100];
 
 	dmn->t_wakeup = NULL;
@@ -846,10 +858,10 @@ static int try_connect(struct daemon *dmn)
 			zlog_debug("%s: connection in progress", dmn->name);
 		dmn->state = DAEMON_CONNECTING;
 		dmn->fd = sock;
-		thread_add_write(master, check_connect, dmn, dmn->fd,
-				 &dmn->t_write);
-		thread_add_timer(master, wakeup_connect_hanging, dmn,
-				 gs.timeout, &dmn->t_wakeup);
+		event_add_write(master, check_connect, dmn, dmn->fd,
+				&dmn->t_write);
+		event_add_timer(master, wakeup_connect_hanging, dmn, gs.timeout,
+				&dmn->t_wakeup);
 		SET_READ_HANDLER(dmn);
 		return 0;
 	}
@@ -860,7 +872,7 @@ static int try_connect(struct daemon *dmn)
 	return 1;
 }
 
-static void phase_hanging(struct thread *t_hanging)
+static void phase_hanging(struct event *t_hanging)
 {
 	gs.t_phase_hanging = NULL;
 	flog_err(EC_WATCHFRR_CONNECTION,
@@ -872,10 +884,10 @@ static void phase_hanging(struct thread *t_hanging)
 static void set_phase(enum restart_phase new_phase)
 {
 	gs.phase = new_phase;
-	thread_cancel(&gs.t_phase_hanging);
+	event_cancel(&gs.t_phase_hanging);
 
-	thread_add_timer(master, phase_hanging, NULL, PHASE_TIMEOUT,
-			 &gs.t_phase_hanging);
+	event_add_timer(master, phase_hanging, NULL, PHASE_TIMEOUT,
+			&gs.t_phase_hanging);
 }
 
 static void phase_check(void)
@@ -906,7 +918,7 @@ static void phase_check(void)
 			"Phased restart: all routing daemon stop jobs have completed.");
 		set_phase(PHASE_WAITING_DOWN);
 
-	/*FALLTHRU*/
+		fallthrough;
 	case PHASE_WAITING_DOWN:
 		if (gs.numdown + IS_UP(gs.special) < gs.numdaemons)
 			break;
@@ -916,7 +928,7 @@ static void phase_check(void)
 			1);
 		set_phase(PHASE_ZEBRA_RESTART_PENDING);
 
-	/*FALLTHRU*/
+		fallthrough;
 	case PHASE_ZEBRA_RESTART_PENDING:
 		if (gs.special->restart.pid)
 			break;
@@ -925,21 +937,18 @@ static void phase_check(void)
 			  gs.special->name);
 		set_phase(PHASE_WAITING_ZEBRA_UP);
 
-	/*FALLTHRU*/
+		fallthrough;
 	case PHASE_WAITING_ZEBRA_UP:
 		if (!IS_UP(gs.special))
 			break;
 		zlog_info("Phased restart: %s is now up.", gs.special->name);
-		{
-			struct daemon *dmn;
-			for (dmn = gs.daemons; dmn; dmn = dmn->next) {
-				if (dmn != gs.special)
-					run_job(&dmn->restart, "start",
-						gs.start_command, 1, 0);
-			}
+		for (dmn = gs.daemons; dmn; dmn = dmn->next) {
+			if (dmn != gs.special)
+				run_job(&dmn->restart, "start",
+					gs.start_command, 1, 0);
 		}
 		gs.phase = PHASE_NONE;
-		THREAD_OFF(gs.t_phase_hanging);
+		EVENT_OFF(gs.t_phase_hanging);
 		zlog_notice("Phased global restart has completed.");
 		break;
 	}
@@ -986,9 +995,9 @@ static void try_restart(struct daemon *dmn)
 	run_job(&gs.restart, "restart", gs.restart_command, 0, 1);
 }
 
-static void wakeup_unresponsive(struct thread *t_wakeup)
+static void wakeup_unresponsive(struct event *t_wakeup)
 {
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 
 	dmn->t_wakeup = NULL;
 	if (dmn->state != DAEMON_UNRESPONSIVE)
@@ -1001,9 +1010,9 @@ static void wakeup_unresponsive(struct thread *t_wakeup)
 	}
 }
 
-static void wakeup_no_answer(struct thread *t_wakeup)
+static void wakeup_no_answer(struct event *t_wakeup)
 {
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 
 	dmn->t_wakeup = NULL;
 	dmn->state = DAEMON_UNRESPONSIVE;
@@ -1016,11 +1025,11 @@ static void wakeup_no_answer(struct thread *t_wakeup)
 	try_restart(dmn);
 }
 
-static void wakeup_send_echo(struct thread *t_wakeup)
+static void wakeup_send_echo(struct event *t_wakeup)
 {
 	static const char echocmd[] = "echo " PING_TOKEN;
 	ssize_t rc;
-	struct daemon *dmn = THREAD_ARG(t_wakeup);
+	struct daemon *dmn = EVENT_ARG(t_wakeup);
 
 	dmn->t_wakeup = NULL;
 	if (((rc = write(dmn->fd, echocmd, sizeof(echocmd))) < 0)
@@ -1032,8 +1041,8 @@ static void wakeup_send_echo(struct thread *t_wakeup)
 		daemon_down(dmn, why);
 	} else {
 		gettimeofday(&dmn->echo_sent, NULL);
-		thread_add_timer(master, wakeup_no_answer, dmn, gs.timeout,
-				 &dmn->t_wakeup);
+		event_add_timer(master, wakeup_no_answer, dmn, gs.timeout,
+				&dmn->t_wakeup);
 	}
 }
 
@@ -1059,6 +1068,8 @@ void watchfrr_status(struct vty *vty)
 	vty_out(vty, " Min Restart Interval: %ld\n", gs.min_restart_interval);
 	vty_out(vty, " Max Restart Interval: %ld\n", gs.max_restart_interval);
 	vty_out(vty, " Restart Timeout: %ld\n", gs.restart_timeout);
+	vty_out(vty, " Reading Configuration: %s\n",
+		gs.reading_configuration ? "yes" : "no");
 	if (gs.restart.pid)
 		vty_out(vty, "    global restart running, pid %ld\n",
 			(long)gs.restart.pid);
@@ -1117,7 +1128,7 @@ static char *translate_blanks(const char *cmd, const char *blankstr)
 	return res;
 }
 
-static void startup_timeout(struct thread *t_wakeup)
+static void startup_timeout(struct event *t_wakeup)
 {
 	daemon_send_ready(1);
 }
@@ -1264,6 +1275,16 @@ static void netns_setup(const char *nsname)
 }
 #endif
 
+static void watchfrr_start_config(void)
+{
+	gs.reading_configuration = true;
+}
+
+static void watchfrr_end_config(void)
+{
+	gs.reading_configuration = false;
+}
+
 static void watchfrr_init(int argc, char **argv)
 {
 	const char *special = "zebra";
@@ -1271,8 +1292,8 @@ static void watchfrr_init(int argc, char **argv)
 	struct daemon *dmn, **add = &gs.daemons;
 	char alldaemons[512] = "", *p = alldaemons;
 
-	thread_add_timer_msec(master, startup_timeout, NULL, STARTUP_TIMEOUT,
-			      &gs.t_startup_timeout);
+	event_add_timer_msec(master, startup_timeout, NULL, STARTUP_TIMEOUT,
+			     &gs.t_startup_timeout);
 
 	for (i = optind; i < argc; i++) {
 		dmn = XCALLOC(MTYPE_WATCHFRR_DAEMON, sizeof(*dmn));
@@ -1282,8 +1303,8 @@ static void watchfrr_init(int argc, char **argv)
 		gs.numdaemons++;
 		gs.numdown++;
 		dmn->fd = -1;
-		thread_add_timer_msec(master, wakeup_init, dmn, 0,
-				      &dmn->t_wakeup);
+		event_add_timer_msec(master, wakeup_init, dmn, 0,
+				     &dmn->t_wakeup);
 		dmn->restart.interval = gs.min_restart_interval;
 		*add = dmn;
 		add = &dmn->next;
@@ -1333,19 +1354,20 @@ static struct frr_signal_t watchfrr_signals[] = {
 	},
 };
 
+/* clang-format off */
 FRR_DAEMON_INFO(watchfrr, WATCHFRR,
-		.flags = FRR_NO_PRIVSEP | FRR_NO_TCPVTY | FRR_LIMITED_CLI
-			 | FRR_NO_CFG_PID_DRY | FRR_NO_ZCLIENT
-			 | FRR_DETACH_LATER,
+	.flags = FRR_NO_PRIVSEP | FRR_NO_TCPVTY | FRR_LIMITED_CLI
+		 | FRR_NO_CFG_PID_DRY | FRR_NO_ZCLIENT | FRR_DETACH_LATER,
 
-		.printhelp = printhelp,
-		.copyright = "Copyright 2004 Andrew J. Schorr",
+	.printhelp = printhelp,
+	.copyright = "Copyright 2004 Andrew J. Schorr",
 
-		.signals = watchfrr_signals,
-		.n_signals = array_size(watchfrr_signals),
+	.signals = watchfrr_signals,
+	.n_signals = array_size(watchfrr_signals),
 
-		.privs = &watchfrr_privs,
+	.privs = &watchfrr_privs,
 );
+/* clang-format on */
 
 #define DEPRECATED_OPTIONS "aAezR:"
 
@@ -1558,6 +1580,7 @@ int main(int argc, char **argv)
 	master = frr_init();
 	watchfrr_error_init();
 	watchfrr_init(argc, argv);
+	cmd_init_config_callbacks(watchfrr_start_config, watchfrr_end_config);
 	watchfrr_vty_init();
 
 	frr_config_fork();
