@@ -1,26 +1,13 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Common ioctl functions.
  * Copyright (C) 1997, 98 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
+#include <sys/ioctl.h>
+
 #include "linklist.h"
 #include "if.h"
 #include "prefix.h"
@@ -403,6 +390,7 @@ int if_unset_prefix_ctx(const struct zebra_dplane_ctx *ctx)
 #endif /* HAVE_STRUCT_IFALIASREQ */
 #endif /* HAVE_NETLINK */
 
+#ifndef HAVE_NETLINK
 /* get interface flags */
 void if_get_flags(struct interface *ifp)
 {
@@ -498,6 +486,7 @@ void if_get_flags(struct interface *ifp)
 out:
 	if_flags_update(ifp, (ifreqflags.ifr_flags & 0x0000ffff));
 }
+#endif
 
 /* Set interface flags */
 int if_set_flags(struct interface *ifp, uint64_t flags)
