@@ -1,32 +1,21 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Main implementation file for interface to Forwarding Plane Manager.
  *
  * Copyright (C) 2012 by Open Source Routing.
  * Copyright (C) 2012 by Internet Systems Consortium, Inc. ("ISC")
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
+#ifdef GNU_LINUX
+#include <linux/rtnetlink.h>
+#endif
+
 #include "log.h"
 #include "libfrr.h"
 #include "stream.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "network.h"
 #include "command.h"
 #include "lib/version.h"
@@ -171,7 +160,7 @@ struct zfpm_glob {
 	 */
 	enum zfpm_msg_format message_format;
 
-	struct thread_master *master;
+	struct event_loop *master;
 
 	enum zfpm_state state;
 
@@ -219,15 +208,15 @@ struct zfpm_glob {
 	/*
 	 * Threads for I/O.
 	 */
-	struct thread *t_connect;
-	struct thread *t_write;
-	struct thread *t_read;
+	struct event *t_connect;
+	struct event *t_write;
+	struct event *t_read;
 
 	/*
 	 * Thread to clean up after the TCP connection to the FPM goes down
 	 * and the state that belongs to it.
 	 */
-	struct thread *t_conn_down;
+	struct event *t_conn_down;
 
 	struct {
 		struct zfpm_rnodes_iter iter;
@@ -237,7 +226,7 @@ struct zfpm_glob {
 	 * Thread to take actions once the TCP conn to the FPM comes up, and
 	 * the state that belongs to it.
 	 */
-	struct thread *t_conn_up;
+	struct event *t_conn_up;
 
 	struct {
 		struct zfpm_rnodes_iter iter;
@@ -266,7 +255,7 @@ struct zfpm_glob {
 	/*
 	 * Stats interval timer.
 	 */
-	struct thread *t_stats;
+	struct event *t_stats;
 
 	/*
 	 * If non-zero, the last time when statistics were cleared.
@@ -284,8 +273,8 @@ static struct zfpm_glob *zfpm_g = &zfpm_glob_space;
 
 static int zfpm_trigger_update(struct route_node *rn, const char *reason);
 
-static void zfpm_read_cb(struct thread *thread);
-static void zfpm_write_cb(struct thread *thread);
+static void zfpm_read_cb(struct event *thread);
+static void zfpm_write_cb(struct event *thread);
 
 static void zfpm_set_state(enum zfpm_state state, const char *reason);
 static void zfpm_start_connect_timer(const char *reason);
@@ -298,9 +287,9 @@ union g_addr ipv4ll_gateway;
 /*
  * zfpm_thread_should_yield
  */
-static inline int zfpm_thread_should_yield(struct thread *t)
+static inline int zfpm_thread_should_yield(struct event *t)
 {
-	return thread_should_yield(t);
+	return event_should_yield(t);
 }
 
 /*
@@ -475,8 +464,8 @@ static inline void zfpm_read_on(void)
 	assert(!zfpm_g->t_read);
 	assert(zfpm_g->sock >= 0);
 
-	thread_add_read(zfpm_g->master, zfpm_read_cb, 0, zfpm_g->sock,
-			&zfpm_g->t_read);
+	event_add_read(zfpm_g->master, zfpm_read_cb, 0, zfpm_g->sock,
+		       &zfpm_g->t_read);
 }
 
 /*
@@ -487,8 +476,8 @@ static inline void zfpm_write_on(void)
 	assert(!zfpm_g->t_write);
 	assert(zfpm_g->sock >= 0);
 
-	thread_add_write(zfpm_g->master, zfpm_write_cb, 0, zfpm_g->sock,
-			 &zfpm_g->t_write);
+	event_add_write(zfpm_g->master, zfpm_write_cb, 0, zfpm_g->sock,
+			&zfpm_g->t_write);
 }
 
 /*
@@ -496,7 +485,7 @@ static inline void zfpm_write_on(void)
  */
 static inline void zfpm_read_off(void)
 {
-	THREAD_OFF(zfpm_g->t_read);
+	EVENT_OFF(zfpm_g->t_read);
 }
 
 /*
@@ -504,12 +493,17 @@ static inline void zfpm_read_off(void)
  */
 static inline void zfpm_write_off(void)
 {
-	THREAD_OFF(zfpm_g->t_write);
+	EVENT_OFF(zfpm_g->t_write);
 }
 
 static inline void zfpm_connect_off(void)
 {
-	THREAD_OFF(zfpm_g->t_connect);
+	EVENT_OFF(zfpm_g->t_connect);
+}
+
+static inline void zfpm_conn_down_off(void)
+{
+	EVENT_OFF(zfpm_g->t_conn_down);
 }
 
 /*
@@ -518,7 +512,7 @@ static inline void zfpm_connect_off(void)
  * Callback for actions to be taken when the connection to the FPM
  * comes up.
  */
-static void zfpm_conn_up_thread_cb(struct thread *thread)
+static void zfpm_conn_up_thread_cb(struct event *thread)
 {
 	struct route_node *rnode;
 	struct zfpm_rnodes_iter *iter;
@@ -557,8 +551,8 @@ static void zfpm_conn_up_thread_cb(struct thread *thread)
 
 		zfpm_g->stats.t_conn_up_yields++;
 		zfpm_rnodes_iter_pause(iter);
-		thread_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb,
-				      NULL, 0, &zfpm_g->t_conn_up);
+		event_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb,
+				     NULL, 0, &zfpm_g->t_conn_up);
 		return;
 	}
 
@@ -583,15 +577,15 @@ static void zfpm_connection_up(const char *detail)
 	/*
 	 * Start thread to push existing routes to the FPM.
 	 */
-	THREAD_OFF(zfpm_g->t_conn_up);
+	EVENT_OFF(zfpm_g->t_conn_up);
 
 	zfpm_rnodes_iter_init(&zfpm_g->t_conn_up_state.iter);
 	zfpm_g->fpm_mac_dump_done = false;
 
 	zfpm_debug("Starting conn_up thread");
 
-	thread_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb, NULL, 0,
-			      &zfpm_g->t_conn_up);
+	event_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb, NULL, 0,
+			     &zfpm_g->t_conn_up);
 	zfpm_g->stats.t_conn_up_starts++;
 }
 
@@ -634,7 +628,7 @@ static void zfpm_connect_check(void)
  * Callback that is invoked to clean up state after the TCP connection
  * to the FPM goes down.
  */
-static void zfpm_conn_down_thread_cb(struct thread *thread)
+static void zfpm_conn_down_thread_cb(struct event *thread)
 {
 	struct route_node *rnode;
 	struct zfpm_rnodes_iter *iter;
@@ -650,8 +644,6 @@ static void zfpm_conn_down_thread_cb(struct thread *thread)
 	while ((mac = TAILQ_FIRST(&zfpm_g->mac_q)) != NULL)
 		zfpm_mac_info_del(mac);
 
-	zfpm_g->t_conn_down = NULL;
-
 	iter = &zfpm_g->t_conn_down_state.iter;
 
 	while ((rnode = zfpm_rnodes_iter_next(iter))) {
@@ -682,9 +674,8 @@ static void zfpm_conn_down_thread_cb(struct thread *thread)
 
 		zfpm_g->stats.t_conn_down_yields++;
 		zfpm_rnodes_iter_pause(iter);
-		zfpm_g->t_conn_down = NULL;
-		thread_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb,
-				      NULL, 0, &zfpm_g->t_conn_down);
+		event_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb,
+				     NULL, 0, &zfpm_g->t_conn_down);
 		return;
 	}
 
@@ -727,9 +718,9 @@ static void zfpm_connection_down(const char *detail)
 	 */
 	assert(!zfpm_g->t_conn_down);
 	zfpm_rnodes_iter_init(&zfpm_g->t_conn_down_state.iter);
-	zfpm_g->t_conn_down = NULL;
-	thread_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb, NULL, 0,
-			      &zfpm_g->t_conn_down);
+	zfpm_conn_down_off();
+	event_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb, NULL, 0,
+			     &zfpm_g->t_conn_down);
 	zfpm_g->stats.t_conn_down_starts++;
 
 	zfpm_set_state(ZFPM_STATE_IDLE, detail);
@@ -738,7 +729,7 @@ static void zfpm_connection_down(const char *detail)
 /*
  * zfpm_read_cb
  */
-static void zfpm_read_cb(struct thread *thread)
+static void zfpm_read_cb(struct event *thread)
 {
 	size_t already;
 	struct stream *ibuf;
@@ -899,11 +890,10 @@ static inline int zfpm_encode_route(rib_dest_t *dest, struct route_entry *re,
 		len = zfpm_netlink_encode_route(cmd, dest, re, in_buf,
 						in_buf_len);
 		assert(fpm_msg_align(len) == len);
-		*msg_type = FPM_MSG_TYPE_NETLINK;
 #endif /* HAVE_NETLINK */
 		break;
 
-	default:
+	case ZFPM_MSG_FORMAT_NONE:
 		break;
 	}
 
@@ -1168,7 +1158,7 @@ static void zfpm_build_updates(void)
 /*
  * zfpm_write_cb
  */
-static void zfpm_write_cb(struct thread *thread)
+static void zfpm_write_cb(struct event *thread)
 {
 	struct stream *s;
 	int num_writes;
@@ -1250,7 +1240,7 @@ static void zfpm_write_cb(struct thread *thread)
 /*
  * zfpm_connect_cb
  */
-static void zfpm_connect_cb(struct thread *t)
+static void zfpm_connect_cb(struct event *t)
 {
 	int sock, ret;
 	struct sockaddr_in serv;
@@ -1404,8 +1394,8 @@ static void zfpm_start_connect_timer(const char *reason)
 	delay_secs = zfpm_calc_connect_delay();
 	zfpm_debug("scheduling connect in %ld seconds", delay_secs);
 
-	thread_add_timer(zfpm_g->master, zfpm_connect_cb, 0, delay_secs,
-			 &zfpm_g->t_connect);
+	event_add_timer(zfpm_g->master, zfpm_connect_cb, 0, delay_secs,
+			&zfpm_g->t_connect);
 	zfpm_set_state(ZFPM_STATE_ACTIVE, reason);
 }
 
@@ -1680,7 +1670,7 @@ static void zfpm_iterate_rmac_table(struct hash_bucket *bucket, void *args)
 /*
  * struct zfpm_statsimer_cb
  */
-static void zfpm_stats_timer_cb(struct thread *t)
+static void zfpm_stats_timer_cb(struct event *t)
 {
 	zfpm_g->t_stats = NULL;
 
@@ -1713,7 +1703,7 @@ static void zfpm_stop_stats_timer(void)
 		return;
 
 	zfpm_debug("Stopping existing stats timer");
-	THREAD_OFF(zfpm_g->t_stats);
+	EVENT_OFF(zfpm_g->t_stats);
 }
 
 /*
@@ -1723,8 +1713,8 @@ void zfpm_start_stats_timer(void)
 {
 	assert(!zfpm_g->t_stats);
 
-	thread_add_timer(zfpm_g->master, zfpm_stats_timer_cb, 0,
-			 ZFPM_STATS_IVL_SECS, &zfpm_g->t_stats);
+	event_add_timer(zfpm_g->master, zfpm_stats_timer_cb, 0,
+			ZFPM_STATS_IVL_SECS, &zfpm_g->t_stats);
 }
 
 /*
@@ -1841,12 +1831,15 @@ DEFUN (clear_zebra_fpm_stats,
 /*
  * update fpm connection information
  */
-DEFUN ( fpm_remote_ip,
+DEFUN (fpm_remote_ip,
        fpm_remote_ip_cmd,
-        "fpm connection ip A.B.C.D port (1-65535)",
-        "fpm connection remote ip and port\n"
-        "Remote fpm server ip A.B.C.D\n"
-        "Enter ip ")
+       "fpm connection ip A.B.C.D port (1-65535)",
+       "Forwarding Path Manager\n"
+       "Configure FPM connection\n"
+       "Connect to IPv4 address\n"
+       "Connect to IPv4 address\n"
+       "TCP port number\n"
+       "TCP port number\n")
 {
 
 	in_addr_t fpm_server;
@@ -1867,13 +1860,16 @@ DEFUN ( fpm_remote_ip,
 	return CMD_SUCCESS;
 }
 
-DEFUN ( no_fpm_remote_ip,
+DEFUN (no_fpm_remote_ip,
        no_fpm_remote_ip_cmd,
-        "no fpm connection ip A.B.C.D port (1-65535)",
-        "fpm connection remote ip and port\n"
-        "Connection\n"
-        "Remote fpm server ip A.B.C.D\n"
-        "Enter ip ")
+       "no fpm connection ip A.B.C.D port (1-65535)",
+       NO_STR
+       "Forwarding Path Manager\n"
+       "Remove configured FPM connection\n"
+       "Connect to IPv4 address\n"
+       "Connect to IPv4 address\n"
+       "TCP port number\n"
+       "TCP port number\n")
 {
 	if (zfpm_g->fpm_server != inet_addr(argv[4]->arg)
 	    || zfpm_g->fpm_port != atoi(argv[6]->arg))
@@ -1988,7 +1984,7 @@ static struct cmd_node zebra_node = {
  *
  * Returns true on success.
  */
-static int zfpm_init(struct thread_master *master)
+static int zfpm_init(struct event_loop *master)
 {
 	int enable = 1;
 	uint16_t port = 0;
@@ -2052,10 +2048,13 @@ static int zfpm_fini(void)
 	zfpm_write_off();
 	zfpm_read_off();
 	zfpm_connect_off();
+	zfpm_conn_down_off();
 
 	zfpm_stop_stats_timer();
 
 	hook_unregister(rib_update, zfpm_trigger_update);
+	hook_unregister(zebra_rmac_update, zfpm_trigger_rmac_update);
+
 	return 0;
 }
 
