@@ -25,12 +25,6 @@ const struct frr_yang_module_info frr_zebra_info = {
 	.name = "frr-zebra",
 	.features = features,
 	.nodes = {
-		{
-			.xpath = "/frr-zebra:zebra/mcast-rpf-lookup",
-			.cbs = {
-				.modify = zebra_mcast_rpf_lookup_modify,
-			}
-		},
 		{
 			.xpath = "/frr-zebra:zebra/ip-forwarding",
 			.cbs = {
