@@ -15,6 +15,7 @@
 #include <sched.h>
 #endif
 #include <dirent.h>
+#include <libgen.h>
 #include <sys/inotify.h>
 #include <sys/stat.h>
 
@@ -41,6 +42,10 @@
 #define ZEBRA_NS_POLLING_INTERVAL_MSEC     1000
 #define ZEBRA_NS_POLLING_MAX_RETRIES  200
 
+#if !defined(__GLIBC__)
+#define basename(src) (strrchr(src, '/') ? strrchr(src, '/') + 1 : src)
+#endif
+
 DEFINE_MTYPE_STATIC(ZEBRA, NETNS_MISC, "ZebraNetNSInfo");
 static struct event *zebra_netns_notify_current;
 
@@ -234,6 +239,7 @@ static void zebra_ns_ready_read(struct event *t)
 {
 	struct zebra_netns_info *zns_info = EVENT_ARG(t);
 	const char *netnspath;
+	const char *netnspath_basename;
 	int err, stop_retry = 0;
 
 	if (!zns_info)
@@ -261,23 +267,24 @@ static void zebra_ns_ready_read(struct event *t)
 		zebra_ns_continue_read(zns_info, stop_retry);
 		return;
 	}
+	netnspath_basename = basename(strdupa(netnspath));
 
 	/* check default name is not already set */
-	if (strmatch(VRF_DEFAULT_NAME, basename(netnspath))) {
-		zlog_warn("NS notify : NS %s is already default VRF.Cancel VRF Creation", basename(netnspath));
+	if (strmatch(VRF_DEFAULT_NAME, netnspath_basename)) {
+		zlog_warn("NS notify : NS %s is already default VRF.Cancel VRF Creation", netnspath_basename);
 		zebra_ns_continue_read(zns_info, 1);
 		return;
 	}
-	if (zebra_ns_notify_is_default_netns(basename(netnspath))) {
+	if (zebra_ns_notify_is_default_netns(netnspath_basename)) {
 		zlog_warn(
 			"NS notify : NS %s is default VRF. Ignore VRF creation",
-			basename(netnspath));
+			netnspath_basename);
 		zebra_ns_continue_read(zns_info, 1);
 		return;
 	}
 
 	/* success : close fd and create zns context */
-	zebra_ns_notify_create_context_from_entry_name(basename(netnspath));
+	zebra_ns_notify_create_context_from_entry_name(netnspath_basename);
 	zebra_ns_continue_read(zns_info, 1);
 }
 
@@ -304,7 +311,7 @@ static void zebra_ns_notify_read(struct event *t)
 		char *netnspath;
 		struct zebra_netns_info *netnsinfo;
 
-		if (!(event->mask & (IN_CREATE | IN_DELETE)))
+		if (!CHECK_FLAG(event->mask, (IN_CREATE | IN_DELETE)))
 			continue;
 
 		if (offsetof(struct inotify_event, name) + event->len
@@ -350,7 +357,7 @@ static void zebra_ns_notify_read(struct event *t)
 		memcpy(event_name, event->name, event->len);
 		event_name[event->len - 1] = 0;
 
-		if (event->mask & IN_DELETE) {
+		if (CHECK_FLAG(event->mask, IN_DELETE)) {
 			zebra_ns_delete(event_name);
 			continue;
 		}
@@ -371,19 +378,28 @@ void zebra_ns_notify_parse(void)
 {
 	struct dirent *dent;
 	DIR *srcdir = opendir(NS_RUN_DIR);
+	int srcdirfd;
 
 	if (srcdir == NULL) {
 		flog_err_sys(EC_LIB_SYSTEM_CALL,
 			     "NS parsing init: failed to parse %s", NS_RUN_DIR);
 		return;
 	}
+
+	srcdirfd = dirfd(srcdir);
+	if (srcdirfd < 0) {
+		closedir(srcdir);
+		flog_err_sys(EC_LIB_SYSTEM_CALL, "NS parsing init: failed to parse %s", NS_RUN_DIR);
+		return;
+	}
+
 	while ((dent = readdir(srcdir)) != NULL) {
 		struct stat st;
 
 		if (strcmp(dent->d_name, ".") == 0
 		    || strcmp(dent->d_name, "..") == 0)
 			continue;
-		if (fstatat(dirfd(srcdir), dent->d_name, &st, 0) < 0) {
+		if (fstatat(srcdirfd, dent->d_name, &st, 0) < 0) {
 			flog_err_sys(
 				EC_LIB_SYSTEM_CALL,
 				"NS parsing init: failed to parse entry %s",
@@ -396,7 +412,7 @@ void zebra_ns_notify_parse(void)
 			continue;
 		}
 		/* check default name is not already set */
-		if (strmatch(VRF_DEFAULT_NAME, basename(dent->d_name))) {
+		if (strmatch(VRF_DEFAULT_NAME, basename(strdupa(dent->d_name)))) {
 			zlog_warn("NS notify : NS %s is already default VRF.Cancel VRF Creation", dent->d_name);
 			continue;
 		}
