@@ -1140,7 +1140,7 @@ int zebra_send_rnh_update(struct rnh *rnh, struct zserv *client,
 	struct stream *s = NULL;
 	struct route_entry *re;
 	unsigned long nump;
-	uint8_t num;
+	uint16_t num;
 	struct nexthop *nh;
 	struct route_node *rn;
 	int ret;
@@ -1211,7 +1211,7 @@ int zebra_send_rnh_update(struct rnh *rnh, struct zserv *client,
 		stream_putl(s, re->metric);
 		num = 0;
 		nump = stream_get_endp(s);
-		stream_putc(s, 0);
+		stream_putw(s, 0);
 
 		nhg = rib_get_fib_nhg(re);
 		for (ALL_NEXTHOPS_PTR(nhg, nh))
@@ -1239,13 +1239,13 @@ int zebra_send_rnh_update(struct rnh *rnh, struct zserv *client,
 				}
 		}
 
-		stream_putc_at(s, nump, num);
+		stream_putw_at(s, nump, num);
 	} else {
 		stream_putc(s, 0); // type
 		stream_putw(s, 0); // instance
 		stream_putc(s, 0); // distance
 		stream_putl(s, 0); // metric
-		stream_putc(s, 0); // nexthops
+		stream_putw(s, 0); // nexthops
 	}
 	stream_putw_at(s, 0, stream_get_endp(s));
 
@@ -1271,7 +1271,7 @@ void show_nexthop_json_helper(json_object *json_nexthop,
 	bool display_vrfid = false;
 	uint8_t rn_family;
 
-	if (re == NULL || nexthop->vrf_id != re->vrf_id)
+	if ((re == NULL || nexthop->vrf_id != re->vrf_id) && nexthop->type != NEXTHOP_TYPE_BLACKHOLE)
 		display_vrfid = true;
 
 	if (rn)
@@ -1292,7 +1292,7 @@ void show_route_nexthop_helper(struct vty *vty, const struct route_node *rn,
 	bool display_vrfid = false;
 	uint8_t rn_family;
 
-	if (re == NULL || nexthop->vrf_id != re->vrf_id)
+	if ((re == NULL || nexthop->vrf_id != re->vrf_id) && nexthop->type != NEXTHOP_TYPE_BLACKHOLE)
 		display_vrfid = true;
 
 	if (rn)
